/*
Copyright 2003/2004/2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Hartmut Ritter

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/

//#include <io.h>
//#include <stdlib.h>

#include "ScatterWeb.Diffusion.h"
#include "ScatterWeb.DiffusionParams.h"
#include "ScatterWeb.DiffusionPtypes.h"
#include "ScatterWeb.DiffusionRouting.h"
#include "ScatterWeb.Event.h"
#include "ScatterWeb.Management.h"
#include "ScatterWeb.DiffusionDataHandling.h"

//externally defined variables
extern unsigned char* command;
extern const struct term_command term_commands[];
extern int debug_level;
extern int localSolarState;
static unsigned char sequence_number;
extern unsigned int interestVector;
extern appconfig_t AppConfig;

//forward declarations
int readGradientTable(unsigned int);
int remote_readGradientTable(unsigned int*);
int do_readGradientTable(int, unsigned int*); 
void sendInitialDataMessage();
int clearGradientTable();
void requestGradientUpdate();
int checkDataInterestMatchSigned (int, unsigned long int);

int yellow_led = 0;
struct GradientStruct GradientTable [GradientTableSize];
int sim_heartbeat_started=0;
int interestValue[8]; //gives value for each interest digit
int interestOperation[8]; //gives operation for each interest digit

void Routing_serialHandler(void* line) {
  UINT8* str = (UINT8*) line;
  //HR added this for DirectedDiffusionRouting
  if(str[0]=='%'){
    int dest, length;    
    char* payload; 
    dest = String_parseInt(&str[1], (UINT8**)&payload);
    payload++; // payload is now on first byte of remote terminal command
    for(length=0; payload[length] != '\0'; length++);
    length++;
    DiffusionRouting_sendRoutable(Configuration.id, dest, MESSAGING_REQ_PACKET, NULL, 0, payload, length, NULL);   
    //orig_rc, final_dest, inner_type, header, header_length, data, data_length, callback
    Comm_clear();      
  }
  //HR end of addition for DirectedDiffusionRouting
  else Messaging_serialHandler();
}

///////////
//Diffusion_init
//handleInterestMesage: Handle incoming packets of type INTEREST_MESSAGE
///////////
void Diffusion_init(){
  sequence_number=1;
  sim_heartbeat_started = 0;
  clearGradientTable(Configuration.id);  
  Comm_print ("\r\n[DBG0]====== ScatterWeb data gathering activated on node %i ===============\r\n", Configuration.id);
  debug_level = DEBUG_VERBOSE;

  DiffusionRouting_init();

  char flags[2];
  flags[0]='1';  
  flags[1]='C';
  AppConfig.announceFlags = String_fromHexToByte( &flags[0]);
  appconfig_save(); 
    
  flags[0]='8';
  flags[1]='9';  
  Net_setTxPower( String_parseInt((char*) &flags[0], NULL)); 
  appconfig_save();
    
  yellow_led = 0;
  
  requestGradientUpdate(40);//look around if neighbours can tell a gradient  
  System_registerCallback(C_SERIAL, Routing_serialHandler);
}

///////////
//consistencyCheck
//check if InterestMessage comes with supported attribute and operation
//returns 1 if o.k., 0 otherwise
///////////
int consistencyCheck(struct InterestMessageStruct InterestMessage)
{
  int passed = 1;
  
  if (InterestMessage.attribute != INTEREST_VIB
      && InterestMessage.attribute != INTEREST_LIGHT
      && InterestMessage.attribute != INTEREST_PIR
      && InterestMessage.attribute != INTEREST_TEMP
      )
  {
    if (debug_level > DEBUG_MEDIUM)
      Comm_print("[DBG3] Interest did not pass consistency check, attribute unknown\r\n");
    passed = 0;  
  }
  if (InterestMessage.operation != INTEREST_OPERATION_GREATER_THAN
      && InterestMessage.operation != INTEREST_OPERATION_SMALLER_THAN
      && InterestMessage.operation != INTEREST_OPERATION_DONT_CARE
      && InterestMessage.operation != INTEREST_OPERATION_GREATER_EQUAL
      && InterestMessage.operation != INTEREST_OPERATION_SMALLER_EQUAL
      )
  {
    if (debug_level > DEBUG_MEDIUM)
      Comm_print("[DBG3]Interest did not pass consistency check, operation unknown \r\n");
    passed = 0;  
  }
  return passed;  
}

///////////
//requestGradientUpdate
//request a gradient update from the neighbours in direct radio connection
//but only if no entry in the gradient table is valid
///////////
void requestGradientUpdate(delay)
{
  struct UpdateMessageStruct UpdateMessage;
  int i = 0;
  UpdateMessage.flags = UPDATE_MESSAGE_FLAG_REQUEST;
    
  //request an update only if our Gradient Table is completely empty
  for (i=0;i<GradientTableSize;i++)            
  {
    if (GradientTable[i].valid)
      return;
  }
  //Net_sendPacket(BROADCAST, UPDATE_MESSAGE, NULL, 0, (char*)&UpdateMessage, sizeof(UpdateMessage), NULL);
  packet_t packet;
  packet.to = BROADCAST;
  packet.from = Configuration.id;
  packet.type = DD_UPDATE_MESSAGE;
  packet.header = NULL;
  packet.header_length = 0;
  packet.data = (char*)&UpdateMessage;
  packet.data_length = sizeof (UpdateMessage);
  Net_send(&packet, NULL);


  if (debug_level > DEBUG_MEDIUM)              
    Comm_print("[DBG3]Update Message::req for update sent \r\n");
  
  Timers_add((delay*200), requestGradientUpdate, delay);
  return;
}

///////////
//sendInterestMessage
//called by insertAndSendNewInterest (as part of the command sim)
//send an Interest Message to destination dest
//returns 1 if successful, otherwise 0
///////////
int sendInterestMessage(struct InterestMessageStruct InterestMessage, unsigned int dest) 
{
  int retval=1;
  Data_redOn();
  InterestMessage.flags &= (~INTEREST_FLAG_LOCAL_INTEREST);//don't forward the flag INTEREST_FLAG_LOCAL_INTEREST!
//  retval = Net_sendPacket(dest, INTEREST_MESSAGE, NULL, 0, (char*)&InterestMessage, (unsigned int) sizeof(InterestMessage), NULL);
  packet_t packet;
  packet.to = dest;
  packet.from = Configuration.id;
  packet.type = DD_INTEREST_MESSAGE;
  packet.header = NULL;
  packet.header_length = 0;
  packet.data = (char*)&InterestMessage;
  packet.data_length = sizeof (InterestMessage);
  retval = Net_send(&packet, NULL);
 
  if (retval != 1)
    retval = 0;
                
  if (debug_level > DEBUG_MEDIUM)
  {              
    if (retval)
      Comm_print("[DBG3]Interest Message sent to %i\r\n", dest);
    else
      Comm_print("[DBG3]InterestMessage could not be sent\r\n");
  }
  Data_redOff();
  return retval;
}

///////////
//handleSimHeartbeat
//regularly called after an Interest was inserted locally
//in order to re-transmit it to the neighbours in direct radio connection
///////////
void handleSimHeartbeat (index)
{
  struct InterestMessageStruct InterestMessage;  
  if (GradientTable[index].valid)  
  {
    InterestMessage.attribute = GradientTable[index].attribute;
    InterestMessage.value = GradientTable[index].value;
    InterestMessage.operation = GradientTable[index].operation;
    InterestMessage.interval = GradientTable[index].interval;
    InterestMessage.heartbeat = GradientTable[index].heartbeat;
    InterestMessage.hop_counter = GradientTable[index].hop_count;
    InterestMessage.flags = INTEREST_FLAG_SIM_HEARTBEAT;
    InterestMessage.sequence_number = sequence_number++;    
    //copy the sequence number into the local Gradient Table Entry
    GradientTable[index].sequence_number = InterestMessage.sequence_number;
    //now send the Interest with the new sequence number
    sendInterestMessage(InterestMessage, BROADCAST);
    if (debug_level > DEBUG_HIGH)
      Comm_print("[DBG4]SimHeartbeat sent\r\n");
  }
  index++;
  if (index == GradientTableSize)
    index = 0;
  Timers_add(30*1000, handleSimHeartbeat, index);
}

/////////////
//insertGradientAt
//called if a new Interest was received and the gradient should be stored
//returns 1 if free entry was found, 0 otherwise
/////////////
int insertGradientAt(int free_entry, struct InterestMessageStruct InterestMessage, unsigned int src)
{
  if (GradientTable[free_entry].valid)
    return 0;//entry was not free!
    
  time_t currentTime;
  Time_getSystemTime(&currentTime);

  //copy Interest into Gradient Table    
  GradientTable[free_entry].attribute = InterestMessage.attribute; 
  GradientTable[free_entry].value = InterestMessage.value; 
  GradientTable[free_entry].operation = InterestMessage.operation;
  GradientTable[free_entry].interval = InterestMessage.interval;
  GradientTable[free_entry].heartbeat = InterestMessage.heartbeat;
  GradientTable[free_entry].lastSendTime = currentTime;
  GradientTable[free_entry].hop_count = InterestMessage.hop_counter;
  GradientTable[free_entry].src = src;
  GradientTable[free_entry].valid = 1;
  GradientTable[free_entry].sequence_number = InterestMessage.sequence_number;
  GradientTable[free_entry].flags = InterestMessage.flags; 

  //make the Interest known on the local node
  DiffusionDataHandling_addSensorInterest(InterestMessage.flags, InterestMessage.attribute, InterestMessage.heartbeat, free_entry);
  
  if (debug_level > DEBUG_MEDIUM)
  {
    Comm_print("[DBG3]new interest added to interestVector ");
    if (InterestMessage.flags & INTEREST_FLAG_SIM_HEARTBEAT)
      Comm_print("based on sim heartbeat message");
    Comm_print("\r\n");
  }     
  return 1;
} 

/////////////
//sequence_numberCheck
//check if a received sequence number is newer than a old one
//(basically, if it larger, but probably there was a wraparound)
//returns 1 if new sequence number seems to be newer, 0 otherwise
/////////////
int sequence_numberCheck(unsigned int old_sequence_number, unsigned int new_sequence_number)
{
  if (debug_level > DEBUG_MEDIUM) 
  {
    Comm_print("seq_.nr. at check: %i",new_sequence_number);
  }

  if (new_sequence_number > old_sequence_number)
    return 1;
  //avoid false negatives at wraparound
  if ((old_sequence_number>65000)&&(new_sequence_number < 100))
    return 1;
  if (debug_level > DEBUG_MEDIUM)
    Comm_print("[DBG3]sequence number check failed\r\n");    
  return 0;
}

/////////////
//changeInterestParamsAt
//change the parameters of an already existing gradient at index i
//returns 1 if entry was found, 0 otherwise
/////////////
int changeInterestParamsAt(int i, struct InterestMessageStruct InterestMessage)
{
  if (GradientTable[i].valid==0)//paranoid, in that case you should have never gone so far...
    return 0;
  if (!consistencyCheck(InterestMessage))
    return 0;    
  if (GradientTable[i].attribute != InterestMessage.attribute)
    return 0;
  if (GradientTable[i].operation != InterestMessage.operation)
    return 0;
  if (!sequence_numberCheck(GradientTable[i].sequence_number, InterestMessage.sequence_number))
    return 0;    
 
  GradientTable[i].value = InterestMessage.value; 
  GradientTable[i].interval = InterestMessage.interval;
  GradientTable[i].sequence_number = InterestMessage.sequence_number;
  GradientTable[i].valid = 1;

  if (debug_level > DEBUG_MEDIUM) 
  {
    Comm_print("[DBG3]seq_nr. at changeInterestParamsAt:");
    Comm_print("%i",InterestMessage.sequence_number);
    Comm_print("resp.");
    Comm_print("%i", GradientTable[i].sequence_number);
  }
    
  if (!(GradientTable[i].flags & INTEREST_FLAG_LOCAL_INTEREST))//not a locally inserted interest 
    interestVector =  interestVector | GradientTable[i].attribute;
    
  if (  InterestMessage.heartbeat
        && GradientTable[i].heartbeat == 0
        &&  (!(GradientTable[i].flags & INTEREST_FLAG_LOCAL_INTEREST)))
    //this means: the new param sets a heartbeat > 0 
    //AND before there was no heartbeat set
    //AND it is not a local Interest
    Timers_add(InterestMessage.heartbeat*1000, handleInterestHeartbeat, i);      

  GradientTable[i].heartbeat = InterestMessage.heartbeat;
  return 1;
}      

/////////////
//changeInterestParams
//called by COMMAND(uim,0)
//change the parameters of an already existing gradient
//therefore find first the related entry in the GradientTable, 
//then call changeInterestParamsAt
//returns 1 if entry was found, 0 otherwise
/////////////                                                      
int changeInterestParams(struct InterestMessageStruct InterestMessage)
{
  int i = 0, modified;
  for (i=0;i<GradientTableSize;i++)  //look for an already existing identical entry in Gradient Table
  {
    if (GradientTable[i].valid==0)
      continue; //skip not valid entries in the first round
    if (GradientTable[i].attribute != InterestMessage.attribute)
      continue; //skip if attribute does not fit 
    if (GradientTable[i].operation != InterestMessage.operation)
      continue; //skip if operation does not fit 
      
    //at this point we found an entry with identical attribute and operation, 
    //we assume that only one fully matching attribute-operation pair exists per Gradient table
    if (debug_level > DEBUG_MEDIUM)
      Comm_print("[DBG3]changeIntParams: Gradient with same attr. and op. found\r\n");
    modified = changeInterestParamsAt(i, InterestMessage);
    return modified;
  }
  return 0;//we did not find a gradient 
}

/////////////
//updateGradientAt
//called by GradientTableLookup()
//change the parameters of an already existing gradient at index i
//a gradient is deemed to be identical if attribute and operation are identical with the Interest
//returns 1 if entry was already known, 0 otherwise (unknown or not identical)
/////////////
int updateGradientAt(int i, struct InterestMessageStruct InterestMessagePacket, unsigned int srcid) 
{
  int already_known = 0;
  if (GradientTable[i].valid==0) 
    return 1;
  if (GradientTable[i].attribute != InterestMessagePacket.attribute)
    return 1;
  if (GradientTable[i].operation != InterestMessagePacket.operation)
    return 1;
  //check if the sequence number is higher, but not if it is a heartbeat message
  if (!(InterestMessagePacket.flags & INTEREST_FLAG_SIM_HEARTBEAT))
  {
    if (GradientTable[i].hop_count < InterestMessagePacket.hop_counter)
      return 1;
  }
  if (
      (GradientTable[i].value == InterestMessagePacket.value)
      && (GradientTable[i].interval == InterestMessagePacket.interval)
      && (GradientTable[i].heartbeat == InterestMessagePacket.heartbeat)
      && (GradientTable[i].sequence_number == InterestMessagePacket.sequence_number)
      )  
    already_known = 1; 
  else 
    already_known = 0;
    

  if (!(InterestMessagePacket.flags & INTEREST_FLAG_SIM_HEARTBEAT))
  //do not update parameters if the Interest Message results from a sim heartbeat
  //those are used only for keeping the gradient direction alive 
  //(params might have been altered directly)
  {
    GradientTable[i].value = InterestMessagePacket.value; 
    GradientTable[i].operation = InterestMessagePacket.operation;
    GradientTable[i].interval = InterestMessagePacket.interval;
  }
  else
  {
    if (debug_level > DEBUG_MEDIUM)
      Comm_print("[DBG3]checking Gradient Entry based on sim heartbeat message\r\n");
  }
  GradientTable[i].sequence_number = InterestMessagePacket.sequence_number;
  GradientTable[i].src = srcid;
  GradientTable[i].hop_count = InterestMessagePacket.hop_counter;
  GradientTable[i].valid = 1;
  GradientTable[i].flags |= InterestMessagePacket.flags; 
    
  if (!(GradientTable[i].flags & INTEREST_FLAG_LOCAL_INTEREST))//not a locally inserted interest 
    interestVector =  interestVector | GradientTable[i].attribute;
    
  if ((!(GradientTable[i].heartbeat))&&(InterestMessagePacket.heartbeat)&&(!(GradientTable[i].flags & INTEREST_FLAG_LOCAL_INTEREST)))
      //the old Gradient did not have a heartbeat set, the new Interest has, and it is not a local Interest
      Timers_add(InterestMessagePacket.heartbeat*1000, handleInterestHeartbeat, i);
  GradientTable[i].heartbeat = InterestMessagePacket.heartbeat;

  return already_known;
}

/////////////
//GradientTableLookup
//called if an Interest was received (handleInterestMessage)
//assumption: only one gradient per attribute-operation pair
//returns 1 if entry was already known, 0 otherwise (unknown or not identical)
/////////////
int GradientTableLookup(unsigned int* dstid, unsigned int* srcid, struct InterestMessageStruct InterestMessagePacket)
//return: 1 if gradient was found
//return: 0 if gradient was not found (gradient is automatically added then)
{
  if (debug_level > DEBUG_MEDIUM)
    Comm_print("[DBG3]GradientTableLookup \r\n");
  if (*dstid ==BROADCAST)
    {}//future DEBUG here...

  int i;
  int matchingEntryFound = 0;
  int already_known=0;

  for (i=0;i<GradientTableSize;i++)
  {
    if (GradientTable[i].valid==0) 
      continue; //skip not valid entries    
    if (GradientTable[i].attribute != InterestMessagePacket.attribute)
      continue; //skip if attribute does not fit 
    if (GradientTable[i].operation != InterestMessagePacket.operation)
      continue; //skip if operation does not fit       
    //for now we assume that only one gradient per attribute-operation-pair is allowed
    if (!sequence_numberCheck(GradientTable[i].sequence_number, InterestMessagePacket.sequence_number))
      return 1;        
    //O.K., this entry seems to fit with both attribute and operation, look if sthg. is new with it

    //do not allow locally inserted interests to be changed from outside
    if (GradientTable[i].flags & INTEREST_FLAG_LOCAL_INTEREST)
      return 1;
    already_known = updateGradientAt(i, InterestMessagePacket, *srcid); 
    matchingEntryFound = 1; 
    break; //we are done 
  }
  if (! matchingEntryFound) //no gradient with similar interest was found, this one is a new one
  {
    for (i=0;i<GradientTableSize;i++)
    {
      if (GradientTable[i].valid==0) 
      break; //we have found a not valid (= free) entry
    }
    insertGradientAt(i, InterestMessagePacket, *srcid);
    //send an initial Data Message to the sink          
    Timers_add(Configuration.id * 1000, sendInitialDataMessage, i);
  }
  if (already_known)//entry was already known
    return 1;
  else
    return 0;
}

/////////////
//forwardGradientTable
//called  by a timer if an updateRequest was received (handleUpdateMessage)
/////////////
void forwardGradientTable(srcid)
{
  int i = 0;
  struct InterestMessageStruct InterestMessage;

  for (i=0;i<GradientTableSize;i++)
  {
    if (!GradientTable[i].valid)
      continue;//skip not valid entries
    if (GradientTable[i].src == (unsigned int) srcid)    
      continue;//dont't tell the requesting nodes gradients that go over the requesting node
      //we could probably even delete this entry then on this node, but this should be done elsewhere      

    InterestMessage.attribute = GradientTable[i].attribute;
    InterestMessage.value = GradientTable[i].value;
    InterestMessage.operation = GradientTable[i].operation;
    InterestMessage.interval = GradientTable[i].interval;
    InterestMessage.heartbeat = GradientTable[i].heartbeat;
    InterestMessage.hop_counter = GradientTable[i].hop_count;
    InterestMessage.sequence_number = GradientTable[i].sequence_number;
    InterestMessage.flags = GradientTable[i].flags;
    
    sendInterestMessage(InterestMessage, srcid);    
    if (debug_level > DEBUG_HIGH)
    {
        Comm_print("[DBG4]sending Interest Message to ");
        Comm_print("%i", (unsigned int) srcid);
        Comm_print("\r\n");        
    }
  }
  return;
}




/////////////
//sendDataMessage
//universial function sending a Data Message
//return 1 if successful, 0 otherwise
/////////////
int sendDataMessage(unsigned int dst, char attribute, unsigned long int result, 
                    char flags, unsigned int source) 
{
  struct DataMessageStruct DataMessage;
  int retval= 1;   
  Data_greenOn();   
  
  DataMessage.attribute = attribute;
  DataMessage.result = result;
  DataMessage.flags = flags;
  DataMessage.src = source;             

  //retval = Net_sendPacket(dst, DATA_MESSAGE, NULL, 0, (char*)&DataMessage, sizeof(DataMessage), NULL);
  packet_t packet;
  packet.to = dst;
  packet.from = Configuration.id;
  packet.type = DD_DATA_MESSAGE;
  packet.header = NULL;
  packet.header_length = 0;
  packet.data = (char*)&DataMessage;
  packet.data_length = sizeof (DataMessage);
  retval = Net_send(&packet, NULL);

  Data_greenOff();                         
  if (retval==1)
    return 1;
  else 
    return 0;
}

////////////////////////////////////////////////////////////////////////////////
// TERMINAL COMMANDS
////////////////////////////////////////////////////////////////////////////////

///////
//insert new interest message generated at this node, implements terminal command sim
///////
int insertAndSendNewInterest(struct InterestMessageStruct InterestMessage, unsigned int requestor)
{  
  int i, free_entry=GradientTableSize+1, result = 0, modified = 0;
  //first check if the values of the Interest Message sound reasonable
  if (!consistencyCheck(InterestMessage))
    return 0;       
  for (i=0;i<GradientTableSize;i++)  //look for an already existing identical entry in Gradient Table
  {
    if (GradientTable[i].valid==0)
    { 
      free_entry = i; //remember a free entry  
      continue; //skip not valid entries in the first round
    }
    if (GradientTable[i].attribute != InterestMessage.attribute)
      continue; //skip if attribute does not fit 
    if (GradientTable[i].operation != InterestMessage.operation)
      continue; //skip if operation does not fit       
    //at this point we found an entry with identical attribute and identical operation
    if (debug_level > DEBUG_MEDIUM)
      Comm_print("[DBG4]local Interest with same attrib. and op. already in Gradient Table\r\n");
    //next, look if we should change the parameters of the existing gradient
    modified = changeInterestParamsAt(i, InterestMessage);      
    //and then forward the Interest Message to all neighbours
    if (modified)
      sendInterestMessage(InterestMessage, BROADCAST);
    return 1;
  }
  //at this point no matching entry was fond  
  if (free_entry > GradientTableSize)
    return 0;//we did not find a free entry, the table is full  
  //at this point we can add a new entry  
  result = insertGradientAt(free_entry, InterestMessage, requestor); 
  if (!result)
  {
    if (debug_level > DEBUG_NO)
      Comm_print("[DBG1]could not insert new interest!\r\n");
    return 0;
  }
  Data_greenOff();
  yellow_led = 1;
  if (debug_level > DEBUG_NO)
    Data_yellowOn();
    
  //and then forward the Interest Message to all neighbours
  sendInterestMessage(InterestMessage, BROADCAST);
  return 1;
}

///////
//clear Gradient Table, implements terminal command cgt (clear gradient table)
///////
int clearGradientTable(int requestor)
{
  int i;
  for (i=0;i<GradientTableSize;i++)
  {
    GradientTable[i].attribute = 0;
    GradientTable[i].value = 0;
    GradientTable[i].operation = 0;
    GradientTable[i].interval = 0;
    GradientTable[i].heartbeat = 0;
    GradientTable[i].hop_count = 0;
    GradientTable[i].lastSendTime = CURRENT_TIME;
    GradientTable[i].flags = 0; 
    GradientTable[i].valid = 0; 
    GradientTable[i].sequence_number = 0;
  }
  interestVector = 0;
  if (requestor == SYSTEM_ID)
    {;}  //sendRS232String("[DBG0]clear gradient table done\r\n"); 
  else
    Comm_print("[RES cgt]clear gradient table done\r\n");
  Data_yellowOn();
  Data_greenOn();
  return 1;
}

////////////////////////////////////////////////////////////////////////////////
//PACKET HANDLING FUNCTIONS
////////////////////////////////////////////////////////////////////////////////

///////////
//handleInterestMesage: Handle incoming packets of type INTEREST_MESSAGE
///////////
int handleInterestMessage(unsigned int* dstid, unsigned int* srcid, char* payload) 
{
   struct InterestMessageStruct* InterestMessage;
   int already_known=0;
   int passed = 0;
   InterestMessage = (struct InterestMessageStruct*) payload;
   //first of all we increment the hop count - and only here!!!
   InterestMessage->hop_counter++;

   passed = consistencyCheck(*InterestMessage);
   if (!passed)
   {
      if (debug_level > DEBUG_HIGH)
      {
        Comm_print("[DBG4]received Interest Message did not pass consistency checks, ignoring Message\r\n"); 
            Comm_print("[DBG4]message dump: \r\n");
            Comm_print("[DBG4]dest ID:");
            Comm_print("%i",*dstid);
            Comm_print(" src ID:");
            Comm_print("%i",*srcid);
            Comm_print("\r\n"); 
            Comm_print("[DBG4]attr "); 
            Comm_print("%i",InterestMessage->attribute);
            Comm_print(" value "); 
            Comm_print("%l",InterestMessage->value);
            Comm_print(" op. "); 
            Comm_print("%i",InterestMessage->operation);
            Comm_print(" int. "); 
            Comm_print("%i",InterestMessage->interval);
            Comm_print(" hops ");
            Comm_print("%i",InterestMessage->hop_counter);
            Comm_print(" seq.nr ");
            Comm_print("%i",InterestMessage->sequence_number);
            Comm_print("\r\n");    
      }
      return 0;   
   }

   if (debug_level > DEBUG_MEDIUM)
   {
    Comm_print("[DBG3]interest message received \r\n");
    Comm_print("[DBG3]dest ID:");
    Comm_print("%i",*dstid);
    Comm_print(" src ID:");
    Comm_print("%i",*srcid);
    Comm_print("\r\n"); 
    Comm_print("[DBG3]attr "); 
    Comm_print("%i",InterestMessage->attribute);
    Comm_print(" value "); 
    //sendRS232LongInt(InterestMessage->value);
    Comm_print("%l",InterestMessage->value);
    Comm_print(" op. "); 
    Comm_print("%i",InterestMessage->operation);
    Comm_print(" int. "); 
    Comm_print("%i",InterestMessage->interval);
    Comm_print(" hops ");
    Comm_print("%i",InterestMessage->hop_counter);
    Comm_print(" seq.nr ");
    Comm_print("%i",InterestMessage->sequence_number);
    Comm_print("\r\n");    
    }
   //
   //lookup in gradient list: 
   //- store if new entry
   //- store if known gradient, but new Interest Message has smaller hop count

   //in one of these cases:
   //- increase hop count and forward
   //else 
   //- discard Interest Message
   
   already_known = GradientTableLookup(dstid, srcid, *InterestMessage);

   if (!already_known)//it was a new Interest (either completely or new params for a known attribute)
   {
    Data_greenOff();
    yellow_led = 1;
    if (debug_level > DEBUG_NO)
      Data_yellowOn();
   }
   
   if ((!already_known)&&(*dstid == BROADCAST))//Interest was new and not only sent to me, forward it to the neighbours
      {
        sendInterestMessage(*InterestMessage, BROADCAST);//forward flags
        if (debug_level > DEBUG_MEDIUM)                 
          Comm_print("[DBG3]Interest Message forwarded \r\n");   
      }
  if (already_known)
  {
    if (debug_level > DEBUG_MEDIUM)                 
      Comm_print("[DBG3]Interest Message was not new \r\n");    
  }
  return 1;
}

////////////
//handleDataMesage: Handle incoming packets of type DATA_MESSAGE
//////////// 
int handleDataMessage(unsigned int* dstid, unsigned int* srcid, char* payload) 
{    
   int found=0;  
   struct DataMessageStruct* DataMessagePacket;
   DataMessagePacket = (struct DataMessageStruct*) payload;

   System_stopWatchdog();
      
   if (debug_level > DEBUG_HIGH)
   {
     Comm_print("[DBG4]Data Message received\r\n");   
     if (DataMessagePacket->flags & DATA_MESSAGE_FLAG_INITIAL)
      Comm_print("[DBG4]type of Data Message: Initial\r\n");
   }
   if (0)//future DEBUG
    Comm_print("%i",*srcid);
    
   if (*dstid != Configuration.id)
    {
      if (debug_level > DEBUG_HIGH)
        Comm_print("[DBG4]DataMessage sniffed (not for me)\r\n");
      System_startWatchdog();
      return 0; 
    }
    
   //use the incoming data for updating the routing table
   DiffusionRouting_updateRoutingTable(DataMessagePacket->src, *srcid); 
     
   found = DiffusionDataHandling_answerInterest (DataMessagePacket->attribute, DataMessagePacket->result, DataMessagePacket->src, DataMessagePacket->flags);//ignoring hop count etc. :-)
   if (debug_level > DEBUG_MEDIUM)
   {
      if ((!found)&&(!(DataMessagePacket->flags & DATA_MESSAGE_FLAG_INITIAL)))
        Comm_print("[DBG3]No Gradient for Data Message found\r\n");
    } 
   System_startWatchdog();
   return 1;
}

///////////
//handleUpdateMesage: Handle incoming packets of type UPDATE_MESSAGE
///////////
int handleUpdateMessage(unsigned int* dstid, unsigned int* srcid, char* payload) 
{
  struct UpdateMessageStruct* UpdateMessagePacket;
  UpdateMessagePacket = (struct UpdateMessageStruct*) payload;

  if (debug_level > DEBUG_HIGH)
  {
     Comm_print("[DBG4]Update Message from ");
     Comm_print("%i",*srcid);
     Comm_print(" received\r\n");   
  }
  if (UpdateMessagePacket->flags == UPDATE_MESSAGE_FLAG_REQUEST)
  {
    if (*dstid == BROADCAST)
      Timers_add(100*Configuration.id, forwardGradientTable,*srcid);
  }
  return 1;
}

////////////////////////////////////////////////////////////////////////////////
//TERMINAL COMMANDS
////////////////////////////////////////////////////////////////////////////////

////////////
//terminal command sim: send interest message
// sim <attribut> <value> <operation> <interval> <heartbeat>
////////////
COMMAND (sim,0)
{  
    if (debug_level > DEBUG_MEDIUM)
    {
      Comm_print("[DBG3]Handling sim command\r\n");
    }        
    if (!sim_heartbeat_started) 
    {
      handleSimHeartbeat(0);//start re-sending Interest Messages
      sim_heartbeat_started = 1;
    }
 
    UINT8* commandptr;
    char attribute = String_parseInt((char*)&str[4], (UINT8**)&commandptr);
    unsigned long int value = String_parseLongInt((char*) ++commandptr, (UINT8**)&commandptr);
    char operation  = String_parseInt((char*) ++commandptr, (UINT8**)&commandptr);
    char interval   = String_parseInt((char*) ++commandptr, (UINT8**)&commandptr); 
    char heartbeat  = String_parseInt ((char*) ++commandptr, (UINT8**)&commandptr); 

    struct InterestMessageStruct InterestMessage;
    InterestMessage.attribute = attribute;
    InterestMessage.value = value;
    InterestMessage.operation = operation;
    InterestMessage.interval = interval;
    InterestMessage.heartbeat = heartbeat;
    InterestMessage.hop_counter = 0;
    InterestMessage.flags = INTEREST_FLAG_LOCAL_INTEREST;
    InterestMessage.sequence_number = sequence_number++;    

    insertAndSendNewInterest(InterestMessage, Configuration.id);
    return;
}

////////////
//terminal command uim: update interest message
////////////
COMMAND(uim, 0)
{
     UINT8* commandptr;

    char attribute = String_parseInt((char*)&str[4], (UINT8**)&commandptr);
    unsigned long int value = String_parseLongInt((char*) &commandptr[6], (UINT8**)&commandptr);
    char operation  = String_parseInt((char*) &commandptr[13], (UINT8**)&commandptr);
    char interval   = String_parseInt((char*) &commandptr[15], (UINT8**)&commandptr); 
    char heartbeat  = String_parseInt ((char*) &commandptr[19], (UINT8**)&commandptr); 
    unsigned int dest = String_parseInt ((char*) &commandptr[23], (UINT8**)&commandptr);

    struct InterestMessageStruct InterestMessage;
    InterestMessage.attribute = attribute;
    InterestMessage.value = value;
    InterestMessage.operation = operation;
    InterestMessage.interval = interval;
    InterestMessage.heartbeat = heartbeat;
    InterestMessage.hop_counter = 0;
    InterestMessage.flags = INTEREST_FLAG_COMMAND;
    InterestMessage.sequence_number = sequence_number++;    
    
    //first case: the update message should be sent to all nodes
    //this is the same as if the command sim was used
    if ((dest == BROADCAST)||(dest == 0))
    {
      insertAndSendNewInterest(InterestMessage, Configuration.id);
      if (debug_level > DEBUG_HIGH) 
        Comm_print("[DBG4]broadcasting Interest update\r\n");
      return;
    }
    //second case: the update message is for this node only  
    if (dest == Configuration.id)
    {
      if (debug_level > DEBUG_MEDIUM)
      {
        Comm_print("[DBG3]Inserting new interest");
      }
      changeInterestParams(InterestMessage);    
      return;
    }
    
    //third case: the update should be forwarded to its non-broadcast destination    
    if (DiffusionRouting_sendRoutable(Configuration.id, dest, DD_INTEREST_MESSAGE, NULL, 0, (unsigned char*)&InterestMessage, (unsigned int) sizeof(InterestMessage), NULL)==1)
    {
      if (debug_level > DEBUG_HIGH) 
      {
        Comm_print("[DBG4]sending routed Interest Message to ");
        Comm_print("%i",dest);
        Comm_print("\r\n");
      }
    }
    else//sending failed because there is no routing table entry for the destiation
    {
      if (debug_level > DEBUG_HIGH)
        Comm_print("[DBG4]could not route the locally inserted Interest Message, no route entry fits\r\n");
      return ;  
    }
}

/////////////
//terminal command rgt: read gradient table
/////////////
COMMAND(rgt,0)
{
    readGradientTable(Configuration.id);    
    return;
}

//
//terminal command cgt: clear gradient table
//
COMMAND (cgt, 0)
{
  clearGradientTable(Configuration.id);
}

///////////////////////////////////////////////////////////////////////////////
//DEBUG FUNCTIONS
///////////////////////////////////////////////////////////////////////////////

////////////
//dumpInterestMessageInfoPacket dumps incoming Interest Message
////////////
int dumpInterestMessageInfoPacket(unsigned int* srcid, struct InterestMessageInfoStruct* InterestMessageInfoPacket)
{
  if (InterestMessageInfoPacket->flags & INTEREST_FLAG_INFORMATIONAL)
    Comm_print("[RES rgt]");
  Comm_print("[ID: ");
  Comm_print("%i",*srcid);
  Comm_print("][ATT: ");
  Comm_print("%i",InterestMessageInfoPacket->attribute);  
  Comm_print("][VAL: "); 
  //sendRS232LongInt((unsigned long int)InterestMessageInfoPacket->value);
  Comm_print("%l",(unsigned long int)InterestMessageInfoPacket->value);
  Comm_print("][OPR: "); 
  Comm_print("%i",(int) InterestMessageInfoPacket->operation);
  Comm_print("][INT: "); 
  Comm_print("%i",(int)InterestMessageInfoPacket->interval);
  Comm_print("][HRT: "); 
  Comm_print("%i",(int)InterestMessageInfoPacket->heartbeat);        
  Comm_print("][HPS: ");
  Comm_print("%i",(int) InterestMessageInfoPacket->hop_counter);
  Comm_print("]");
  if (InterestMessageInfoPacket->flags & INTEREST_FLAG_INFORMATIONAL)
  {
    Comm_print("[SRC: ");
    Comm_print("%i",(unsigned int) InterestMessageInfoPacket->src);
    Comm_print("]");
  }  
  Comm_print("\r\n");    
  return 1;
}

////////////
//do_readGradientTable dumps complete Gradient Table
////////////
int do_readGradientTable(int remote, unsigned int* dstid) //implements terminal command rgt
{
  int i;
  System_stopWatchdog();

  if ((!remote)&& (*dstid != Configuration.id))//in that case send answer back as routable packets
  {
    struct InterestMessageInfoStruct InterestMessageInfo;
    for (i=0;i<GradientTableSize;i++)
    {
      if (!GradientTable[i].valid) continue; //skip invalid gradient table entries
      
      InterestMessageInfo.attribute = GradientTable[i].attribute;
      InterestMessageInfo.value = GradientTable[i].value;
      InterestMessageInfo.operation = GradientTable[i].operation;
      InterestMessageInfo.interval = GradientTable[i].interval;
      InterestMessageInfo.heartbeat = GradientTable[i].heartbeat;
      InterestMessageInfo.hop_counter = GradientTable[i].hop_count;
      InterestMessageInfo.sequence_number = GradientTable[i].sequence_number;
      InterestMessageInfo.flags = INTEREST_FLAG_INFORMATIONAL;
      InterestMessageInfo.src = GradientTable[i].src;

      if (!DiffusionRouting_sendRoutable(Configuration.id, *dstid, DD_INTEREST_MESSAGE, NULL, 0, (unsigned char*)&InterestMessageInfo, (unsigned int) sizeof(InterestMessageInfo), NULL))
      {
        System_startWatchdog();
        return 0;    
      }
    }     
    System_startWatchdog();
    return 1;
  }  

    if (remote)
    {
      sendMgmtString("\r\n==== gradient table dump of node ", MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtLongInt(Configuration.id, MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtString(" ====\r\n", MGMT_TYPE_RGT_REPLY, dstid);
    }
    else
    {
      Comm_print("\r\n[RES rgt][ID: ");
      Comm_print("%i",Configuration.id);
      Comm_print("]\r\n");
    }
      
  for (i=0;i<GradientTableSize;i++)
  {
    if (!GradientTable[i].valid) continue; //skip invalid gradient table entries
    if (remote)
      sendMgmtString("\r\nattr.:", MGMT_TYPE_RGT_REPLY, dstid);
    else
      Comm_print("\r\n[RES rgt][ATT: "); 
    switch (GradientTable[i].attribute)
    {      
      case INTEREST_VIB:
        if (remote)
          sendMgmtString("VIB", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print("VIB]");
        break;
      case INTEREST_LIGHT:
        if (remote)
          Comm_print("LIGHT", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print("LUM]"); 
       break;
      case INTEREST_PIR:
        if (remote)
          sendMgmtString("PIR", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print("PIR]"); 
        break;
      case INTEREST_TEMP:
        if (remote)
          sendMgmtString("TMP", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print("TMP]"); 
        break;
      default:
        if (remote)
        {
          Comm_print("unknown attribute", MGMT_TYPE_RGT_REPLY, dstid);
          sendMgmtInt(GradientTable[i].attribute, MGMT_TYPE_RGT_REPLY, dstid);
        }
        else
        {
          Comm_print ("???"); 
          Comm_print("%i",GradientTable[i].attribute);
          Comm_print ("]"); 
        }
    }
    if (remote)
      sendMgmtString(" op:", MGMT_TYPE_RGT_REPLY, dstid);
    else
      Comm_print("[OPR: ");
    switch (GradientTable[i].operation)
    {      
      case INTEREST_OPERATION_GREATER_THAN:
        if (remote)
          sendMgmtString(">", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print(">]");
        break;
      case INTEREST_OPERATION_SMALLER_THAN:
        if (remote)
          sendMgmtString("<", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print("<]");
        break;
      case INTEREST_OPERATION_DONT_CARE:
        if (remote)
          sendMgmtString("x]", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print("x");
        break;
      case INTEREST_OPERATION_GREATER_EQUAL:
        if (remote)
          sendMgmtString(">=", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print(">=]");
        break;
      case INTEREST_OPERATION_SMALLER_EQUAL:
        if (remote)
          sendMgmtString("<=", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print("<=]");
        break;
      default:
        if (remote)
        {
          sendMgmtString("unknown op.", MGMT_TYPE_RGT_REPLY, dstid);
          sendMgmtInt(GradientTable[i].operation, MGMT_TYPE_RGT_REPLY, dstid);
        }
        else
        {
          Comm_print("?");
          Comm_print("%i",GradientTable[i].operation);    
          Comm_print("]");
        }
    }
    if (remote)
    {
      sendMgmtString(" value:", MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtLongInt(GradientTable[i].value, MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtString(" interv.:", MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtLongInt(GradientTable[i].interval, MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtString(" src:", MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtLongInt(GradientTable[i].src, MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtString(" hops:", MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtLongInt(GradientTable[i].hop_count, MGMT_TYPE_RGT_REPLY, dstid);
//      sendMgmtString(" flags:", MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtString(" heartbeat:", MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtLongInt(GradientTable[i].heartbeat, MGMT_TYPE_RGT_REPLY, dstid);      
    }
    else
    {
      Comm_print("[VAL: ");
      //sendRS232LongInt(GradientTable[i].value); 
      Comm_print("%l",GradientTable[i].value);
      Comm_print("][INT: ");
      Comm_print("%i",GradientTable[i].interval); 
      Comm_print("][SRC: ");
      Comm_print("%i",GradientTable[i].src);
      Comm_print("][HPS: "); 
      Comm_print("%i",GradientTable[i].hop_count); 
      Comm_print("][HRT: "); 
      Comm_print("%i",GradientTable[i].heartbeat); 
      Comm_print("][FLG: ");
    }
    if (GradientTable[i].flags & INTEREST_FLAG_LOCAL_INTEREST)
      {
        if (remote)
          sendMgmtString("LOCAL_INTEREST", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print("LOCAL_INTEREST]");
      }
    if (remote)
    {
      sendMgmtString(" seq.nr. ", MGMT_TYPE_RGT_REPLY, dstid);
      sendMgmtInt(GradientTable[i].sequence_number, MGMT_TYPE_RGT_REPLY, dstid);
    }
    else
    {
      Comm_print("[SEQ: "); 
      //sendRS232LongInt(GradientTable[i].sequence_number);
      Comm_print("%i",GradientTable[i].sequence_number);
      Comm_print("]");
    }
  }
  if (remote)
  {
    sendMgmtString("\r\nInterest Vector: ", MGMT_TYPE_RGT_REPLY, dstid);
    sendMgmtLongInt(interestVector, MGMT_TYPE_RGT_REPLY, dstid);
  }
  else
  {  
    Comm_print("\r\n[RES rgt][ID: ");
    Comm_print("%i",Configuration.id);
    Comm_print("]");
    Comm_print("[VEC: ");
    //sendRS232Int(interestVector);
  }
  if (interestVector & INTEREST_VIB)
  {
        if (remote)
          sendMgmtString(" VIB ", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print(" VIB "); 
  }
  if (interestVector & INTEREST_LIGHT)
  {
        if (remote)
          sendMgmtString(" LIGHT ", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print(" LUM "); 
  }
  if (interestVector & INTEREST_PIR)
  {
        if (remote)
          sendMgmtString(" PIR ", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print(" PIR "); 
  }
  if (interestVector & INTEREST_TEMP)
  {
        if (remote)
          sendMgmtString(" TMP ", MGMT_TYPE_RGT_REPLY, dstid);
        else
          Comm_print(" TMP"); 
  }
  if (remote)
    sendMgmtString(" \r\n", MGMT_TYPE_RGT_REPLY, dstid);
  else
    Comm_print("]\r\n"); 
  System_startWatchdog();
  return 1;
}


int readGradientTable(unsigned int requestor)     //simple wrapper to set the output direction
{
  do_readGradientTable(0, &requestor);//not a direct remoting command, but either
  //send answer over RS232 or as a routable packet back to the requestor
  return 0;
}

int remote_readGradientTable(unsigned int* dstid)
{
    do_readGradientTable(1, dstid);//we want to answer to a remote rgt command
    return 0;
}




////////////////////////////////////////////////////////////////////////////////
// HELPER FUNCTIONS
////////////////////////////////////////////////////////////////////////////////

int copyInterestMessageInfoToInterestMessage(struct InterestMessageStruct* InterestMessagePacket, struct InterestMessageInfoStruct* InterestMessageInfoPacket)
{
  InterestMessagePacket->attribute       =InterestMessageInfoPacket->attribute;
  InterestMessagePacket->value           =InterestMessageInfoPacket->value;
  InterestMessagePacket->operation       =InterestMessageInfoPacket->operation;
  InterestMessagePacket->interval        =InterestMessageInfoPacket->interval;
  InterestMessagePacket->heartbeat       =InterestMessageInfoPacket->heartbeat;
  InterestMessagePacket->hop_counter     =InterestMessageInfoPacket->hop_counter;
  InterestMessagePacket->flags           =InterestMessageInfoPacket->flags;
  InterestMessagePacket->sequence_number =InterestMessageInfoPacket->sequence_number;
  return 1;
}

void handleButtonClick() 
{
  if (debug_level == DEBUG_NO)
    debug_level = DEBUG_HIGH;
  else
    debug_level = DEBUG_NO;
}

bool Diffusion_routableRequestHandler (unsigned int dest, unsigned char* payload) {
  packet_t packet;
  packet.to = dest;  
  packet.from = Configuration.id;
  packet.type = MESSAGING_ANS_PACKET;
  packet.header = 0;   
  packet.header_length = 0;
  Messaging_doCommand(payload, 0);  
  // Count length and send data  
  packet.data = term_reply;         
  for(packet.data_length=0; packet.data[packet.data_length] != '\0'; packet.data_length++);
  packet.data_length++;
  DiffusionRouting_sendRoutablePacket(&packet, 0);     
  return True;
}

bool Diffusion_routableAnswerHandler(unsigned int orig, unsigned char* payload) {
  Comm_print("[%i] %s\r\n", orig, payload);
  return True;
}

