/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Hartmut Ritter

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/

#ifndef ScatterWeb_Diffusion
#define ScatterWeb_Diffusion

#include "../../../../System/src/ScatterWeb.System.h"

#define INTEREST_VIB 1 //interest in vibration sensor data
#define INTEREST_LIGHT 2 //interest in light sensor data
#define INTEREST_PIR 4 //interest in movement (PIR sensor)
#define INTEREST_TEMP 8 //interest in temperature

#define INTEREST_OPERATION_GREATER_THAN 1
#define INTEREST_OPERATION_SMALLER_THAN 2
#define INTEREST_OPERATION_DONT_CARE 3
#define INTEREST_OPERATION_GREATER_EQUAL 4
#define INTEREST_OPERATION_SMALLER_EQUAL 5

#define INTEREST_FLAG_INFORMATIONAL 1 //answer to rgt terminal command
#define INTEREST_FLAG_COMMAND 2 //issued by uim terminal command
#define INTEREST_FLAG_SIM_HEARTBEAT 4 //message results from interest heartbeat
#define INTEREST_FLAG_LOCAL_INTEREST 8 //flag in the gradient table denoting local interest (interest was inserted at this node)

#define DATA_MESSAGE_FLAG_DATA 0
#define DATA_MESSAGE_FLAG_INITIAL 1

#define UPDATE_MESSAGE_FLAG_REQUEST 1
#define UPDATE_MESSAGE_FLAG_COMMAND 2

#define GradientTableSize 10

struct InterestMessageStruct {
  char attribute;//kind of sensor 
  unsigned long int value;//sensed value
  char operation;//>, <, <= etc.
  unsigned int interval;//peak rate of event reporting
  unsigned int heartbeat;//rate of reporting state
  char hop_counter;//no of hops
  char flags;
  unsigned char sequence_number;
};

struct InterestMessageInfoStruct {
  char attribute;//kind of sensor 
  unsigned long int value;//sensed value
  char operation;//>, <, <= etc.
  unsigned int interval;//peak rate of event reporting
  unsigned int heartbeat;//rate of reporting state
  char hop_counter;//no of hops
  char flags;
  unsigned char sequence_number;
  unsigned int src;
};

struct InterestMessageCommandStruct {
  char valid;//state of the entry: valid or not
  char attribute;//kind of sensor 
  unsigned long int value;//sensed value
  char operation;//>, <, <= etc.
  unsigned int interval;//peak rate of event reporting
  unsigned int heartbeat;//rate of reporting state
  char flags;
  unsigned char sequence_number;
  unsigned int src;
  unsigned int dest;//BROADCAST or id in case of term_uim
  unsigned int timestamp;
};

struct GradientStruct {
  char valid;
  char attribute;
  unsigned long int value;
  char operation;  
  unsigned int interval;
  unsigned int heartbeat;
  char hop_count;
  unsigned int src;
//   unsigned int alt_src;
  char flags;//(bit 0: solar state, 1=sun, 0=no sun)  
  //unsigned int lastSendTime;
  time_t lastSendTime;
  unsigned char sequence_number;
};

struct DataMessageStruct {
  char attribute;
  unsigned long int result;
  char flags;
  unsigned int src;
};

struct UpdateMessageStruct {
  char flags;
};

void Diffusion_init();
int sendInterestMessage(struct InterestMessageStruct, unsigned int);
//InterestMessage, dest
int insertAndSendNewInterest(struct InterestMessageStruct, unsigned int);//insert new insert message from this node
//InterestMEssage requestor (local or one hop away like USB stick)
int handleInterestMessage(unsigned int*, unsigned int*, char*);
//dstid, srcid, payload
int sendDataMessage(unsigned int, char, unsigned long int, char, unsigned int);   
//destination, attribute, result, flags, source
int handleDataMessage(unsigned int*, unsigned int*, char*);
//dstid, srcid, payload
int answerInterest(char, unsigned long int, unsigned int, char);
//attribute, result, orig, data flags
int handleUpdateMessage(unsigned int*, unsigned int*, char*); 
//dstid, srcid, payload
int changeInterestParams(struct InterestMessageStruct);
//InterestMessage
int GradientTableLookup(unsigned int*, unsigned int*, struct InterestMessageStruct); 
int readGradientTable();    
int remote_readGradientTable(unsigned int*);
int clearGradientTable();
int insertGradientAt(int, struct InterestMessageStruct, unsigned int);
//entry, InterestMessage, src
int nextHopFromGradientTable(unsigned int*);
//id
int consistencyCheck(struct InterestMessageStruct);
//InterestMessage
int dumpInterestMessageInfoPacket(unsigned int*, struct InterestMessageInfoStruct*);
//sricd, InterestMessageInfo
int copyInterestMessageInfoToInterestMessage(struct InterestMessageStruct*, struct InterestMessageInfoStruct*);


void term_cgt();
void handleButtonClick();

/* 
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
void Diffusion_serialHandler();

//HR added this for handling routed MESSAGING_REQ_PACKETS
/*
 * Function to handle request that came in a routed packet
 * 
 * @param unsigned int    ID where request came from originally 
 * @param unsigned char*  pointer to payload 
 * @return bool           True always  
 *  
 */ 
bool Diffusion_routableRequestHandler (unsigned int, unsigned char*);
//HR addition ends 
 
//HR added this for handling routed MESSAGING_ANS_PACKETS
/*
 * Function to handle answer that were routed to this node
 * @param unsigned int    ID where request came from originally 
 * @param unsigned char*  pointer to payload 
 *  
 * @return bool   True always  
 *  
 */
bool Diffusion_routableAnswerHandler(unsigned int, unsigned char*);
//HR addition ends

#endif
