/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Hartmut Ritter

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/

#include <io.h>
#include "../../../../System/src/ScatterWeb.System.h"

#include "ScatterWeb.Diffusion.h"
#include "ScatterWeb.DiffusionParams.h"
#include "ScatterWeb.DiffusionPtypes.h"
#include "ScatterWeb.DiffusionRouting.h"
#include "ScatterWeb.Event.h"
#include "ScatterWeb.Management.h"


#include <stdlib.h>

int discovery_state;
int discovery_reason;

extern int debug_level;

char intbuf[8];
int setUnsignedInt(unsigned int hex){
  char tbuf[8];
  int i;
  int value;
  int counter = 0;

  if(hex<=0){ tbuf[0] = '0'; counter=1; }
  else {
    while(hex >= 1){
    
   	value = hex % 10;
    	hex  /= 10;
   	tbuf[counter] =  '0' + value;
  	counter++;
    }  	  
  }
  for(i=0; i<counter; i++){
    intbuf[i] = tbuf[counter-1-i];
  }
  intbuf[counter] = '\0';
  
  return counter;  
}

char longintbuf[10];
int setUnsignedLongInt(unsigned long int hex){
  char tbuf[10];
  int i;
  unsigned long int value;
  int counter = 0;

  if(hex<=0){ tbuf[0] = '0'; counter=1; }
  else {
    while(hex >= 1){
    
   	value = hex % 10;
    	hex  /= 10;
   	tbuf[counter] =  '0' + value;
  	counter++;
    }  	  
  }
  for(i=0; i<counter; i++){
    longintbuf[i] = tbuf[counter-1-i];
  }
  longintbuf[counter] = '\0';
  
  return counter;  
}

unsigned int writeUnsignedIntFixed(unsigned int value, char* c, unsigned int num){
  int i = num;
  unsigned int t = value;
  
  while(i>0){
    i--;
    c[i] = (t % 10) + 0x30;
    t /= 10;    
  }
  return num;
}


int sendMgmtPongPacket(unsigned int, unsigned int, unsigned int);//dstid, remote, originator
int sendMgmtFlashlightPacket(unsigned int dstid);

int sendMgmtChars(unsigned char* data, int n, char type, unsigned int* dstid)
{
    int i=0,j=0;
    struct MgmtPacketStruct MgmtPacket;
    //unsigned char transmissionPower = Configuration.transmitPower;
    unsigned char transmissionPower = Configuration.transmitPower;

    MgmtPacket.type = type; 
    Net_setTxPower(99);      

    MgmtPacket.size = MGMT_TYPE_REPLY_MAXSIZE;   
    for (i=0;i<n;i++)
    {
     MgmtPacket.data[j]=data[i];
     j++;
     if (j==MGMT_TYPE_REPLY_MAXSIZE)
     {
       //(unsigned char to, unsigned char type,
       //         unsigned char num, unsigned int length, char* data)
                
//        sendPacket(*dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

//      sendPacket_noCD(*dstid,MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);


          packet_t p;        
          p.to = *dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) (&MgmtPacket);
          p.data_length = sizeof(MgmtPacket);
//          Net_send(&p, NULL);
          Net_send(&p, NULL);        

        j=0;
//        System_wait(1000);
     }             
    }
    if (j>0) //send the rest in a not fully filled packet
    {
      MgmtPacket.size = j;
      //sendPacket(*dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);  
//      sendPacket_noCD(*dstid,MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);


          packet_t p;          
          p.to = *dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) (&MgmtPacket);
          p.data_length = sizeof(MgmtPacket); 
//          Net_send(&p, NULL);
          Net_send(&p, NULL);
      
      
    }
    Net_setTxPower(transmissionPower);
    return 0;
}

int sendMgmtString(unsigned char* string, char type, unsigned int* dstid)
{
  int size=0;
  while (string[size])
      size++;
  if (debug_level > DEBUG_HIGH)
  {
    Comm_print("sending a mgmt remote reply packet to ");
    Comm_print("%i",*dstid);
    Comm_print("\r\n");
  }
  sendMgmtChars(string, size, type, dstid);
  return 0;
}

int sendMgmtInt(unsigned int value, char type, unsigned int* dstid)
{ 
  sendMgmtChars(&intbuf[0], setUnsignedInt(value), type, dstid);
  return 0;
}

int sendMgmtLongInt(unsigned long int value, char type, unsigned int* dstid)
{ 
  sendMgmtChars(&longintbuf[0], setUnsignedLongInt(value), type, dstid);
  return 0;
}

int handleMgmtRgtReplyPacket(unsigned int* dstid, unsigned int* srcid, struct MgmtPacketStruct* MgmtPacket)
{
  if (*dstid != Configuration.id)
    return 0;  
  if (0)
    Comm_print("%i",*srcid);
/*
  if (debug_level == DEBUG_VERBOSE)
      {
        Comm_print("got a mgmt reply packet from node ");
        Comm_print("%i",srcid);
        Comm_print("\r\n");
      } 
*/
//  sendRS232Chars(MgmtPacket->data, MgmtPacket->size);  
    Comm_print("%c",MgmtPacket->data);
    Comm_print("%c",MgmtPacket->size);
  return 1;
}

int handleMgmtCgtPacket(unsigned int* dstid, unsigned int* srcid)
{
  if ((*dstid == Configuration.id)|| (*dstid==BROADCAST))
  {
    if (debug_level > DEBUG_MEDIUM)
    {
      Comm_print("got a mgmt req to exec cgt ");
      if (debug_level == DEBUG_VERBOSE)
      {
        Comm_print("from node ");
        Comm_print("%i",*srcid);
      }
      Comm_print("\r\n");
    } 
    clearGradientTable();
  }
  else {;}
  return 0;
}

int handleMgmtRgtPacket(unsigned int* dstid, unsigned int* srcid)
{
  if (*dstid == Configuration.id)
  {
    if (debug_level > DEBUG_MEDIUM)
    {
      Comm_print("got a mgmt req to exec rgt ");
      if (debug_level == DEBUG_VERBOSE)
      {
        Comm_print("from node ");
        Comm_print("%i",*srcid);
      }
        Comm_print("\r\n");
    }
    remote_readGradientTable(srcid);
  }
  else {;}
  return 0;
}

int handlePingPacket(unsigned int* dstid, unsigned int* srcid)
{
  if (*dstid == Configuration.id)
  {
    if (debug_level > DEBUG_NO)
    {
      if (debug_level > DEBUG_MEDIUM)
      {
        Comm_print("got a ping packet from node ");
        Comm_print("%i",*srcid);
        Comm_print("\r\n");
      }
      Data_greenOn();
      System_wait(10000);
      Data_greenOff();
    }
  }
  else
  {
    if (debug_level > DEBUG_NO)
    {
      if (debug_level > DEBUG_MEDIUM)
      {
        Comm_print("got a ping packet not for me, but for node ");
        Comm_print("%i",*dstid);
        Comm_print(", sent from ");
        Comm_print("%i",*srcid); 
        Comm_print("\r\n");
      }
      Data_redOff();
      Data_beeperOn(500);
      System_wait(10000);
      Data_redOff();
    }
  }
  return 0;
}

int handlePingAckdPacket(unsigned int* dstid, unsigned int* srcid, unsigned int originator, unsigned int final_dest)
{ 
  if (*dstid != Configuration.id)
    {
      if (*dstid == BROADCAST)
      {
        if (debug_level > DEBUG_HIGH)
        {
        Comm_print("answering broadcast ping from");
        Comm_print("%i",*srcid);
        }
        System_wait (4000*Configuration.id);//System_wait to make sure not all nodes answer simultaneously
        sendMgmtPongPacket(*srcid, 0, 0);//send back pong packet to: srcid, no originator, no final_dest
        return 0;  
      }
      else //not for me, not a BROADCAST ping
        return 1;
    }
  if (!originator)
  {
    if (debug_level > DEBUG_HIGH)
    {
      Comm_print("answering normal ping from");
      Comm_print("%i",srcid);
    }
    sendMgmtPongPacket(*srcid, 0, 0);//send back pong packet to: srcid, no originator, no final_dest
    return 0;
  }
  if (final_dest != Configuration.id)
  {
    MgmtPacket.type = MGMT_TYPE_PING_ACKD;
    MgmtPacket.data[0] = originator;
    MgmtPacket.data[1] = final_dest;
    if (debug_level > DEBUG_HIGH)
    {
      Comm_print("forwarding remote ping to ");
      Comm_print("%i",final_dest);
      Comm_print(" on behalf of ");
      Comm_print("%i",originator);
      Comm_print("\r\n");
    }
 //   sendPacket(final_dest, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);
 
         packet_t p;
          
          p.to = final_dest;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) (&MgmtPacket);
          p.data_length = sizeof(MgmtPacket);
  
//          Net_send(&p, NULL);
          Net_send(&p,NULL);
 
 
 
    return 0;
  }
  if (final_dest == Configuration.id)
  {
    if (debug_level > DEBUG_HIGH)
    {
      Comm_print("answering remote ping that arrived from");
      Comm_print("%i",srcid);
      Comm_print(" on behalf of ");
      Comm_print("%i",originator);
      Comm_print("\r\n");
    } 
    sendMgmtPongPacket(*srcid, originator, final_dest);//send back to srcid, tell the real originator
    return 0;
  }
  return 1;
}

//int handlePongPacket(unsigned long int* dstid, unsigned* srcid, unsigned int originator, unsigned char final_dest)
int handlePongPacket(unsigned int* dstid, unsigned int* srcid, unsigned int originator, unsigned int final_dest)
{
  if (*dstid == Configuration.id)
  {
    if (originator)
    {
      unsigned char transmissionPower = Configuration.transmitPower;
      
      if (debug_level > DEBUG_HIGH)
      {
        Comm_print("got a remote pong packet from");
        Comm_print("%i",final_dest);
        Comm_print(" anwering a ping from ");
        Comm_print("%i",originator);
        Comm_print("\r\n");
      }
      Net_setTxPower(99);
      sendMgmtString("got a pong packet from ", MGMT_TYPE_RGT_REPLY, &originator);
      sendMgmtLongInt(final_dest, MGMT_TYPE_RGT_REPLY, &originator);
      sendMgmtString("\r\n", MGMT_TYPE_RGT_REPLY, &originator); 
      Net_setTxPower(transmissionPower);
    }
    else
    { 
      Comm_print("got a normal pong packet from ");
      Comm_print("%i",srcid);
      Comm_print("\r\n");
      
      //HR commented the following out during quick port to ESB/2
      //if (discovery_state)//we are in the state of neighbour discovery
        //addNeighbourToList(*srcid); 
    }  
  }
  return 0;
}

int handleMgmtRSTPacket(unsigned int* dstid, unsigned int* srcid)
{
  if (*srcid == Configuration.id)
    return 1;//avoid loops
  if ((*dstid == Configuration.id)|| (*dstid==BROADCAST))
  {
    if (debug_level > DEBUG_MEDIUM)
    {
      Comm_print("got a mgmt req from ");
      Comm_print("%i",srcid);
      Comm_print(" to exec RST\r\n");
    }
    System_reset();
  }
  else {;}
  return 0;
}

int handleMgmtDbgLvlPacket(unsigned int* dstid, unsigned int* srcid, unsigned int dbglevel)
{
  if (*srcid == Configuration.id)
    return 1;//avoid loops
  if (debug_level > DEBUG_HIGH)
  {
    Comm_print("debug level set to ");
    Comm_print("%i",dbglevel);
    if (*dstid != Configuration.id)
     Comm_print("though not for me only");
    Comm_print("\r\n");
  }
  debug_level = dbglevel;
  return 0;
}

int handleMgmtStpPacket(unsigned int* dstid, unsigned int* srcid, unsigned int transmissionpower)
{
  if (debug_level == DEBUG_VERBOSE)
    {
      Comm_print("got a stp req from ");
      Comm_print("%i",srcid);
      Comm_print("\r\n");
    } 
  if ((*dstid == Configuration.id)|| (*dstid==BROADCAST))
  {
    if (transmissionpower > 99)
      transmissionpower = 99;
    Net_setTxPower(transmissionpower);//note that we do not save this in the eeprom
    return 0;
  }
  else return 1;
}

int handleMgmtRtpPacket(unsigned int* dstid, unsigned int* srcid)
{
    unsigned char transmissionPower = Configuration.transmitPower;
    
    if (*dstid == Configuration.id)
    {      
      Net_setTxPower(99);
      sendMgmtString("transm. power of node ", MGMT_TYPE_RTP_REPLY, srcid);
      sendMgmtLongInt(Configuration.id, MGMT_TYPE_RTP_REPLY, srcid);
      sendMgmtString(" is ", MGMT_TYPE_RTP_REPLY, srcid); 
      sendMgmtLongInt(transmissionPower, MGMT_TYPE_RTP_REPLY, srcid);
      sendMgmtString("\r\n", MGMT_TYPE_RTP_REPLY, srcid);  
      Net_setTxPower(transmissionPower);
      return 0;
    }
    return 1;
}

int handleMgmtRtpReplyPacket(unsigned int* dstid, unsigned int* srcid, struct MgmtPacketStruct* MgmtPacket)
{
  if (*dstid != Configuration.id)
    return 0; 
  if (debug_level == DEBUG_VERBOSE)
    {
      Comm_print("got a rtp reply from ");
      Comm_print("%i",srcid);
      Comm_print("\r\n");
    }   
  Comm_print("%c",MgmtPacket->data);  
  Comm_print("%c",MgmtPacket->size);  
  return 1;
}

int handleMgmtRvsPacket(unsigned int* dstid, unsigned int* srcid)
{
    unsigned char transmissionPower = Configuration.transmitPower;
    char reply[6];
    reply[3]='\r';
    reply[4]='\n';
    reply[5]=0;
                                               
    if ((*dstid == Configuration.id)||(*dstid == BROADCAST))
    {      
      Net_setTxPower(99);
      
      // converts an int into a string, result is written to *c
//int writeUnsignedInt(unsigned int hex, char* c){
  
  // second int conversion routine for a fixed number of digits
// same as before, but maximum number of digits/bytes written is num		
//unsigned int writeUnsignedIntFixed(unsigned int value, char* c, unsigned int num){
  //return int i = num; 
   	//sendReply( writeUnsignedIntFixed( sensorData.vibCount, (char*) reply, 3));    	

      //writeUnsignedIntFixed(sensorData.vibCount, reply, 3);
      //writeUnsignedIntFixed(261, reply, 3);
      sendMgmtString(reply,MGMT_TYPE_RVS_REPLY, srcid);
      //sendMgmtLongInt(sensorData.vibCount, MGMT_TYPE_RVS_REPLY, srcid);      
      Net_setTxPower(transmissionPower);
      return 0;
    }
    return 1;
}

int handleMgmtRvsReplyPacket(unsigned int* dstid, unsigned int* srcid, struct MgmtPacketStruct* MgmtPacket)
{
  if ((*dstid != Configuration.id)&&(*dstid !=BROADCAST))
    return 0;   
  if (debug_level == DEBUG_VERBOSE)
    {
      Comm_print("got a rvs reply from ");
      Comm_print("%i",srcid);
      Comm_print("\r\n");
    } 
//  sendRS232Chars(MgmtPacket->data, MgmtPacket->size);  
  Comm_print("%c", MgmtPacket->data);  
  Comm_print("%c", MgmtPacket->size);  
  return 1;
}

int handleMgmtDntPacket(unsigned int* dstid, unsigned int* srcid)
{
    unsigned char transmissionPower = Configuration.transmitPower;
    if (debug_level == DEBUG_VERBOSE)
    {
      Comm_print("got a dnt req from ");
      Comm_print("%i",srcid);
      Comm_print("\r\n");
    } 
    if (*dstid == Configuration.id)
    {      
      Net_setTxPower(99);
      sendMgmtString("Neighbour table is going to be determined\r\n", MGMT_TYPE_DNT_REPLY, srcid);
      Net_setTxPower(transmissionPower);
      //determineDirectNeighbours(1, srcid, NULL);      HR commented out during port to ESB/2
      return 0;
    }
    return 1;
}

int handleMgmtRntPacket(unsigned int* dstid, unsigned int* srcid)
{
    unsigned char transmissionPower = Configuration.transmitPower;
    if (debug_level == DEBUG_VERBOSE)
    {
      Comm_print("got a rnt req from ");
      Comm_print("%i",srcid);
      Comm_print("\r\n");
    } 
    if (*dstid == Configuration.id)
    {      
      Net_setTxPower(99);
      sendMgmtString("Neighbour table of node ", MGMT_TYPE_RNT_REPLY, srcid);
      sendMgmtLongInt(Configuration.id, MGMT_TYPE_RNT_REPLY, srcid);
      sendMgmtString("\r\n", MGMT_TYPE_RNT_REPLY, srcid); 
      Net_setTxPower(transmissionPower);
      //readNeighbourTable(1, srcid); HR commented out during port to ESB/2
      return 0;
    }
    return 1;
}

int handleMgmtCntPacket(unsigned int* dstid, unsigned int* srcid)
{ 
    if ((*dstid == Configuration.id)|| (*dstid==BROADCAST))
    {      
      //clearNeighbourTable(1, srcid); HR commented out during port to ESB/2
      return 0;
    }
    if (debug_level == DEBUG_VERBOSE)
    {
      Comm_print("got a cnt req from ");
      Comm_print("%i",srcid);
      Comm_print("\r\n");
    } 
    return 1;
}

int handleMgmtDntReplyPacket(unsigned int* dstid, unsigned int* srcid, struct MgmtPacketStruct* MgmtPacket)
{
  if (*dstid != Configuration.id)
    return 0;   
  if (debug_level == DEBUG_VERBOSE)
    {
      Comm_print("got a dnt reply from ");
      Comm_print("%i",srcid);
      Comm_print("\r\n");
    } 
//  sendRS232Chars(MgmtPacket->data, MgmtPacket->size);  
  Comm_print("%c",MgmtPacket->data);
  Comm_print("%c", MgmtPacket->size);
  return 1;
}

/*
int handleMgmtFlashPassPacket(unsigned int* dstid, unsigned int* srcid)//, struct MgmtPacketStruct* MgmtPacket)
{
  unsigned int neighbour=0;
  Data_yellowOn();
  Data_greenOn();
  Data_redOff();
  System_wait(1000);
  if (*dstid != Configuration.id)
    return 0;   
  if (debug_level == DEBUG_VERBOSE)
    {
      Comm_print("got a flash pass req from ");
      Comm_print("%i",srcid);
      Comm_print("\r\n");
    } 
  if (discovery_state==DISC_STATE_DONE)
    {
    //neighbour=nextNeighbour((char*)MgmtPacket);//find "good" next neighbour HR commented out during port to ESB/2
    sendMgmtFlashlightPacket(neighbour);//send flashlight request to the neighbour
    }
  else
  {
    discovery_reason=DISC_REASON_FLASH_REMOTEREQ;
    //HR commented out during quick port to ESB/2
    //determineDirectNeighbours(0,0, (char*)MgmtPacket);  
  }
  Data_redOff();
  Data_greenOff();
  YELLOWOFF;    
  return 1;
}
*/

int handleMgmtPacket() 
{ 
  packet_t p=rxPacket; 
  
  //int k=0;
  //??????
  
  //radioReceive(&p);
  
//  unsigned char* dstid;
//  unsigned char* srcid;
  unsigned int* dstid;
  unsigned int* srcid;
  char* payload;
  srcid = &(p.from);
  dstid = &(p.to);
  payload = p.data;
  //payload = (struct sensorDataRecord*)p.data;

  struct MgmtPacketStruct* MgmtPacket;
  MgmtPacket = (struct MgmtPacketStruct*) payload;
  
  switch (MgmtPacket->type)
  {
    case MGMT_TYPE_PING:
      handlePingPacket(dstid, srcid);
      break;
    case MGMT_TYPE_PING_ACKD:
      handlePingAckdPacket(dstid, srcid, MgmtPacket->data[0], MgmtPacket->data[1]);
      break; 
    case MGMT_TYPE_PONG:
      handlePongPacket(dstid, srcid, MgmtPacket->data[0], MgmtPacket->data[1]);
      break;
    case MGMT_TYPE_RGT:
      handleMgmtRgtPacket(dstid, srcid);
      break;
    case MGMT_TYPE_CGT:
      handleMgmtCgtPacket(dstid, srcid);
      break;
    case MGMT_TYPE_RGT_REPLY:
      handleMgmtRgtReplyPacket(dstid, srcid, MgmtPacket);
      break; 
    case MGMT_TYPE_RST:
      handleMgmtRSTPacket(dstid, srcid);
      break;
    case MGMT_TYPE_DBGLVL:
      handleMgmtDbgLvlPacket(dstid, srcid, MgmtPacket->data[0]);
      break;
    case MGMT_TYPE_STP:
      handleMgmtStpPacket(dstid, srcid, MgmtPacket->data[0]);
      break;
    case MGMT_TYPE_RTP:
      handleMgmtRtpPacket(dstid, srcid);
      break;      
    case MGMT_TYPE_RTP_REPLY:
      handleMgmtRtpReplyPacket(dstid, srcid, MgmtPacket);
      break;       
    case MGMT_TYPE_RVS:
      handleMgmtRvsPacket(dstid, srcid);
      break;       
    case MGMT_TYPE_RVS_REPLY:
      handleMgmtRvsReplyPacket(dstid, srcid, MgmtPacket);
      break;         
    case MGMT_TYPE_DNT:
      handleMgmtDntPacket(dstid, srcid);
      break;
    case MGMT_TYPE_RNT:
      handleMgmtRntPacket(dstid, srcid);
      break;
    case MGMT_TYPE_CNT:
      handleMgmtCntPacket(dstid, srcid);
      break;
    case MGMT_TYPE_DNT_REPLY:
    case MGMT_TYPE_RNT_REPLY:
    case MGMT_TYPE_CNT_REPLY:
      handleMgmtDntReplyPacket(dstid, srcid, MgmtPacket);
      break;       
//    case MGMT_TYPE_FLASH_PASS:
//      handleMgmtFlashPassPacket(dstid, srcid);//, MgmtPacket);
//      break;
    case MGMT_TYPE_RAW_REPLY:
      //sendRS232Chars(MgmtPacket->data, MgmtPacket->size);  
      Comm_print("%c",MgmtPacket->data);
      Comm_print("%c",MgmtPacket->size);  
      break;
    default:
      if (debug_level > DEBUG_MEDIUM)
        Comm_print("rcvd a currently not implemented MGMT packet type\r\n");
      break;
  }
  return (1);
}

int sendMgmtPongPacket(unsigned int dstid, unsigned int originator, unsigned int final_dest) 
{
  MgmtPacket.type = MGMT_TYPE_PONG;
  MgmtPacket.data[0] = originator;
  MgmtPacket.data[1] = final_dest;
  //sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);
  
          packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
//          Net_send(&p, NULL);
          Net_send(&p, NULL);  
  
  
  if (debug_level > DEBUG_HIGH)
  {
    Comm_print("MgmtPongPacket sent back to ");
    Comm_print("%i",dstid);
    Comm_print("\r\n");
  }
  return 1;
}

int sendMgmtPingPacket(unsigned int dstid, unsigned int type, unsigned int final_dest)
{
  MgmtPacket.type = type;
  unsigned char transmissionPower = Configuration.transmitPower;
  if (final_dest)
  {
    Net_setTxPower(99);
    MgmtPacket.data[0] = Configuration.id;//originator of remote ping request
    MgmtPacket.data[1] = final_dest;//final destination
    if (debug_level > DEBUG_HIGH)
    {
      Comm_print("sending remote ping; requesting ");
      Comm_print("%i",dstid);
      Comm_print(" to ping ");
      Comm_print("%i",final_dest);
      Comm_print("\r\n");
    }
  }
  else 
  {
    MgmtPacket.data[0] = 0;//clear the field to indicate it was local
    if (debug_level > DEBUG_HIGH)
    {
      Comm_print("sending normal local ping; requesting ");
      Comm_print("%i",dstid);
      Comm_print(" to answer\r\n");
    }
  } 
  //void sendPacket(unsigned char to, unsigned char type,
  //              unsigned char num, unsigned int length, char* data){
  
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket); 
//          Net_send(&p, NULL);
          Net_send(&p, NULL);


  Net_setTxPower(transmissionPower);
  return 1;

}

int sendMgmtRSTPacket(unsigned int dstid)
{
  unsigned char transmissionPower = Configuration.transmitPower;
  Net_setTxPower(99);
  MgmtPacket.type = MGMT_TYPE_RST; 
  MgmtPacket.size = 0;
  if (dstid==0)
    dstid = BROADCAST; 
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);

  Net_setTxPower(transmissionPower);
  return 0;
}

int sendMgmtCgtPacket(unsigned int dstid)
{
  MgmtPacket.type = MGMT_TYPE_CGT;
  MgmtPacket.size = 0;
  if (dstid==0)
    dstid = BROADCAST;
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) (&MgmtPacket);
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);
  return 0;
}

int sendMgmtRgtPacket(unsigned int dstid)
{
  MgmtPacket.type = MGMT_TYPE_RGT;
  MgmtPacket.size = 0;
  if (dstid==0)
  {
    if (debug_level > DEBUG_NO)
    Comm_print("remote RGT to Broadcast address denied \r\n");
    return 1;
   }    
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);


  return 0;
}

int sendMgmtDntPacket(unsigned int dstid)
{
  unsigned char transmissionPower = Configuration.transmitPower;
  Net_setTxPower(99);
  MgmtPacket.type = MGMT_TYPE_DNT;
  MgmtPacket.size = 0;
  if (dstid==0)
  {
    if (debug_level > DEBUG_NO)
    Comm_print("remote DNT to Broadcast address denied \r\n");
    return 1;
   }    
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);


  Net_setTxPower(transmissionPower);
  return 0;
}

int sendMgmtRntPacket(unsigned int dstid)
{
  unsigned char transmissionPower = Configuration.transmitPower;
  Net_setTxPower(99);
  MgmtPacket.type = MGMT_TYPE_RNT;
  MgmtPacket.size = 0;
  if (dstid==0)
  {
    if (debug_level > DEBUG_NO)
    Comm_print("remote RNT to Broadcast address denied \r\n");
    return 1;
   }    
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);



  Net_setTxPower(transmissionPower);
  return 0;
}

int sendMgmtCntPacket(unsigned int dstid)
{
  unsigned char transmissionPower = Configuration.transmitPower;
  Net_setTxPower(99);
  MgmtPacket.type = MGMT_TYPE_CNT;
  MgmtPacket.size = 0;
  if (dstid==0)
  {
    if (debug_level > DEBUG_NO)
    Comm_print("attempt to clear all neighbour tables \r\n");
   }    
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);



  Net_setTxPower(transmissionPower);
  return 0;
}

int sendMgmtDbglvlPacket(unsigned int dstid, unsigned int dbglevel)//dstid, dbg level
{
  unsigned char transmissionPower = Configuration.transmitPower;
  Net_setTxPower(99);
  MgmtPacket.type = MGMT_TYPE_DBGLVL; 
  MgmtPacket.size = 1;
  MgmtPacket.data[0]=dbglevel;
  if (dstid==0)
    dstid = BROADCAST; 
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);



  Net_setTxPower(transmissionPower);
  return 0;
}

int sendMgmtStpPacket(unsigned int dstid, unsigned int newPower)
{
  unsigned char transmissionPower = Configuration.transmitPower;
  Net_setTxPower(99);
  MgmtPacket.type = MGMT_TYPE_STP; 
  MgmtPacket.size = 1;
  MgmtPacket.data[0]=newPower;
  if (dstid==0)
    dstid = BROADCAST; 
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);



  Net_setTxPower(transmissionPower);
  return 0;
}

int sendMgmtRtpPacket(unsigned int dstid)
{
  unsigned char transmissionPower = Configuration.transmitPower;
  Net_setTxPower(99);
  MgmtPacket.type = MGMT_TYPE_RTP; 
  MgmtPacket.size = 0;

  if (dstid==BROADCAST)
  {
    if (debug_level > DEBUG_NO)
    Comm_print("remote Rtp to Broadcast address denied \r\n");
    return 1;
   }  
//  sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);


        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);


  Net_setTxPower(transmissionPower);
  return 0;
}

int sendMgmtRvsPacket()
{
  unsigned char transmissionPower = Configuration.transmitPower;
  Net_setTxPower(99);
  MgmtPacket.type = MGMT_TYPE_RVS; 
  MgmtPacket.size = 0;

//  sendPacket(BROADCAST, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = BROADCAST;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);



  Net_setTxPower(transmissionPower);
  return 0;
}

int sendMgmtFlashlightPackettoNextNeighbour()
{                                             
    unsigned int dstid=0;  
    //HR commented out during quick port to ESB/2
    //dstid=nearestNeighbour();
    if (!dstid)
      return -1;
    sendMgmtFlashlightPacket(dstid);
    return 0;
}

int sendMgmtFlashlightPacket(unsigned int dstid)
{
    MgmtPacket.type = MGMT_TYPE_FLASH_PASS;
    MgmtPacket.size = 1;
    MgmtPacket.data[0]= Configuration.id;
//    sendPacket(dstid, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);

        packet_t p;
          
          p.to = dstid;
          p.type = DD_MGMT_PACKET;
          p.header = 0;
          p.header_length = 0;
          p.data = (char*) &MgmtPacket;
          p.data_length = sizeof(MgmtPacket);
  
          Net_send(&p, NULL);



    if (debug_level > DEBUG_NO)
      Comm_print("Button Click: MGMT_TYPE_FLASH_PASS sent \r\n");
    return 0;
}



/*void handleButtonClick() 
{
  MgmtPacket.type = MGMT_TYPE_PING;
  sendPacket(BROADCAST, Configuration.id, MGMT_PACKET, 0, sizeof(MgmtPacket), (char*) &MgmtPacket);
  if (debug_level > DEBUG_NO)
    Comm_print("Button Click: MgmtPacket sent \r\n");
}
*/
  
/*
void handleButtonClick() 
{ 
  //find best neighbour
  //send flashlight packet to neighbour
  
  if (discovery_state ==DISC_STATE_DONE)
  {
    discovery_reason=DISC_REASON_FLASH_LOCALREQ; 
    Data_yellowOn();
    Data_greenOn();
    Data_redOff();
    System_wait(1000);
    Data_redOff();
    Data_greenOff();
    YELLOWOFF;
    sendMgmtFlashlightPackettoNextNeighbour();
  }
  else//neighbour discovery not yet done
  {
    discovery_reason=DISC_REASON_FLASH_LOCALREQ;
    discovery_state=DISC_STATE_PENDING;
    //HR commented out during quick port to ESB/2
    //determineDirectNeighbours(0,0, NULL);
  }
    
  
}
*/
