/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Hartmut Ritter

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/

#ifndef ScatterWeb_Management
#define ScatterWeb_Management

#include "../../../../System/src/ScatterWeb.System.h"



#define MGMT_TYPE_REPLY_MAXSIZE 10

#define MGMT_TYPE_PING 0
#define MGMT_TYPE_PING_ACKD 1
#define MGMT_TYPE_PONG 2
#define MGMT_TYPE_CGT 3
#define MGMT_TYPE_RGT 4
#define MGMT_TYPE_RGT_REPLY 5
#define MGMT_TYPE_RST 6
#define MGMT_TYPE_DBGLVL 7
#define MGMT_TYPE_STP 8
#define MGMT_TYPE_RTP 9
#define MGMT_TYPE_RTP_REPLY 10
#define MGMT_TYPE_DNT 11 
#define MGMT_TYPE_DNT_REPLY 12
#define MGMT_TYPE_RNT 13 
#define MGMT_TYPE_RNT_REPLY 14
#define MGMT_TYPE_CNT 15 
#define MGMT_TYPE_CNT_REPLY 16
#define MGMT_TYPE_FLASH_PASS 17
#define MGMT_TYPE_RVS 18
#define MGMT_TYPE_RVS_REPLY 19
#define MGMT_TYPE_RAW_REPLY 20

volatile struct MgmtPacketStruct {
  char type;
  char size; //size of data if present
  char data[MGMT_TYPE_REPLY_MAXSIZE];  
}MgmtPacket;

int sendMgmtString(unsigned char*, char, unsigned int*); // string, type, dstid
int sendMgmtInt(unsigned int, char, unsigned int*);//value, type, dstid
int sendMgmtLongInt(unsigned long int, char, unsigned int* );//value, type, dstid
int sendMgmtCgtPacket(unsigned int);//node ID, 0 = BROADCAST
int sendMgmtRgtPacket(unsigned int);//node ID, 0 = BROADCAST
int handleMgmtPacket ();

int sendMgmtRSTPacket(unsigned int);//dstid
int sendMgmtDbglvlPacket(unsigned int, unsigned int);//dstid, dbg level
int sendMgmtStpPacket (unsigned int, unsigned int);//dstid, transmission power
int sendMgmtRtpPacket(unsigned int);//dstid
int sendMgmtDntPacket(unsigned int); //dstid
int sendMgmtRntPacket(unsigned int); //dstid
int sendMgmtCntPacket(unsigned int); //dstid

int sendMgmtRvsPacket(); //remote reading of vibration sensor

int sendMgmtFlashlightPackettoNextNeighbour();
int sendMgmtFlashlightPacket(unsigned int);//dstid

int sendMgmtPingPacket(unsigned int, unsigned int, unsigned int); //dest id, type, remote (node ID) or not
int handlePingPacket(unsigned int*, unsigned int*);//dstid, srcid




#endif
