/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

/** \file ScatterWeb.MMS.h
 ** \ingroup Applications
 ** \brief 1. Introduction
 ** This interface will support sending information (especially video data) from
 ** the sensor network via mobile phones for watching and controlling the
 ** ScatterWeb environment over large distances. In particular, receiving video
 ** data taken by camera nodes of the network has been implemented for Siemens
 ** S65 and might be sent outside the local sensorweb via MMS in upcoming mobile
 ** phones supporting MIDP 2.0 with JSR-205 [2] extensions.
 ** <p> 
 ** 2. Using
 ** Features:
 ** <ul> 
 ** <li>Picture sending
 ** </ul> 
 ** <p>
 ** Preparation:
 ** <ul> 
 ** <li>Connect the ESB to a cellular phone (tested with Siemens S65)
 ** <li>Connect a second ESB to the camera module
 ** <li>Start the java mobile application
 ** </ul> 
 ** <p>
 ** MMS send:
 ** <ul> 
 ** <li>Such as PictureIt! application
 ** </ul> 
 ** <p>
 ** Note:
 ** <ul> 
 ** <li>MMS will be standardised in JSR-205 at GCF
 ** <li>VCP is standardised in J2ME for optional use
 ** </ul> 
 ** <p>  
 ** 3. Requirements
 ** Mobile phones must support:
 ** <ul>
 ** <li>J2ME MIDP 2.0
 ** <li>serial interterface access (CommConnection)
 ** <li>JSR-205 Wireless Messaging API
 ** </ul>
 ** <p>
 ** JSR-205 is not supported by mobile phones at the time this project was
 ** realized but it will replace JSR-120 (WMA 1.1) one day. WMA 1.1 already
 ** makes it possible to send SMS from your Java MIDlet but no multimedia data.
 ** Serial interface access is not possible with all phones as far as the
 ** CommConnection interface is not part of the MIDP 2.0 standard.
 ** <p>
 ** Architecture and Implementation
 ** For video data messages a sensor node with a camera module is needed and
 ** positioned due to its supervising purpose.
 ** <p>
 ** Taking pictures can be arranged by sending a PICTURE_REQ_PACKET from a
 ** common ESB node (for example when sensor values are changing) to the camera
 ** node but it can also be requested by the phone through the serial line but
 ** requests to the camera node have to happen using a radio packet as the
 ** camera node itself is using its own serial line internally.
 ** <p>
 ** Its possible to request JPG formatted pictures of sizes 80x64, 160x128,
 ** 320x240 and 640x480. The taken picture is send as PICTURE_ANS_PACKET to the
 ** demanding node. If this node is connected to a mobile phone, the video data
 ** is forwarded to the serial line.
 ** <p>
 ** Video data is sent line by line at a maximum size of 64 bytes each. Every
 ** received line is forwarded to the mobile phone over the RS232 interface. A
 ** new line consists the following information:
 ** <ul>
 ** <li>1st byte : hi byte of line ID
 ** <li>2nd byte : lo byte of line ID
 ** <li>3rd byte : hi byte of length of line (could be skipped)
 ** <li>4th byte : lo byte of length of line
 ** <li>5th up to 68th byte : jpeg formatted data
 ** </ul>
 ** <p>
 ** Length of line describes size of raw data only. ID starts with 0 from first
 ** line, increases with each line and ID of last line is 0xF0F0.
 ** <p>
 ** The Java MIDlet running on the cell phone (or other compliant devices) has
 ** to handle those lines and put it together to a JPEG file.  
 **
 ** @author        Marco Hutta
 ** @version       2.2
 ** @since         2.2
 **/
    
#ifndef ScatterWeb_MMS
#define ScatterWeb_MMS

#include "../../../../System/src/ScatterWeb.System.h"

/**
 * Handles incoming MMS
 *
 * @return        <code>True</code> always.
 * @since         2.2
 */
bool MMS_handler();

#endif
