/*
 * RS232.java
 *#
 * Created on 1. Dezember 2004, 14:16
 */

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Vector;

import CommTest.*;


/**
 *
 * @author  schran
 * @version
 */
public class RS232Test extends MIDlet implements CommandListener 
{
    private Command     exitCommand;    // exit command
    private Command     menuCommand;    // back to main menu command
    private Command     setupCommand;   // go to setup screen

    private Display     display;        // The display for this MIDlet
    private Form        mainForm;
    private Form        imageForm;
    private List        menu;
    private TextBox     errorBox;
    private ImageItem   imageItem = null;

    private RS232Connection conn;
    private int     comPort         = 0;
    private String  cameraNodeID    = "999";
    private int     picResolution   = 1;


    
    public RS232Test() 
    {
        display     = Display.getDisplay(this);
        
        exitCommand = new Command("Exit", Command.SCREEN, 2);
        menuCommand = new Command("Back to menu", Command.SCREEN, 2);
        setupCommand = new Command("Back", Command.SCREEN, 2);
        
        imageForm = new Form("Last Picture");
        imageForm.addCommand(menuCommand);
        imageForm.setCommandListener(this);

        errorBox = new TextBox("Error", "", 256, 0);
        errorBox.addCommand(exitCommand);
        errorBox.setCommandListener(this);

        menu = new List("ScatterCell Menu",  Choice.IMPLICIT, new String[]{"Setup","Get Picture!","Terminal","Quit"}, null);
        menu.setCommandListener(this);

        
        display.setCurrent(menu);
    }

    public boolean testConnection(RS232Connection conn, int comPort)
    {
        if (!(conn.open(comPort, "115200"))) return false;
        if (!(conn.openInputStream())) return false;
        if (!(conn.openOutputStream())) return false;
        if (!(conn.closeInputStream())) return false;
        if (!(conn.closeOutputStream())) return false;
        if (!(conn.close())) return false;
        
        return true;
    }

    public byte[] getPicture()
    {
        SerialEval se = new SerialEval();

        conn.open(0, "115200");
        conn.openInputStream();
        conn.openOutputStream();
        
        try {Thread.sleep(1000);} catch (Exception e) { }
        conn.execCommand("pic "+cameraNodeID+" "+picResolution);

        boolean done = false;
        
        while (!done)
        {
            // are we receiving data?
            int n = conn.inputAvailable();

            if (n > 0)
            {
                for (int i = 0; i < n; i++)
                {
                    int b = conn.readInt();
                    se.eval(b);
                }
            }
            
            if (se.error()) 
            {
                conn.close();
                conn.closeInputStream();
                conn.closeOutputStream();
                return null;
            }
            
            // if we are done and no error occured
            if (se.picReady())
            {
                conn.close();
                conn.closeInputStream();
                conn.closeOutputStream();
                done = true;
                return se.getPicture();
            }
        }
        
        conn.close();
        conn.closeInputStream();
        conn.closeOutputStream();
        return null;
    }
    
    public void startApp() 
    {
        conn = new RS232Connection();
        if (!testConnection(conn, comPort))
        {
            errorBox.setString("Serial Connection could not be established.");
            display.setCurrent(errorBox);
            conn.closeAll();
        }
        else
        {
            while (true)
            {
                // do some stuff
            }
        }
/*        
        try {Thread.sleep(1000);} catch (Exception e) { }
        conn.execCommand("swr");
        try {Thread.sleep(1000);} catch (Exception e) { }
        conn.execCommand("pic 999 2");
        try {Thread.sleep(2000);} catch (Exception e) { }
*/

    }
    
    public void pauseApp() 
    {
    }
    
    public void destroyApp(boolean unconditional) 
    {
    }

    /*
     * Respond to commands, including exit
     * On the exit command, cleanup and notify that the MIDlet has been destroyed.
     */
    public void commandAction(Command c, Displayable s) {
        if (c == exitCommand) 
        {
            destroyApp(false);
            notifyDestroyed();
        }
        else if (c == menuCommand) 
        {
            display.setCurrent(menu);
        }
        else if (c == List.SELECT_COMMAND)
        {
            switch (menu.getSelectedIndex())
            {
                case 0:
                    final List setupMenu;
                    setupMenu = new List("Setup",  Choice.IMPLICIT, new String[]{"Pic Resolution","Camera Node ID","Commport"}, null);
                    final Command back = new Command("Back", Command.SCREEN, 2);
                    setupMenu.addCommand(back);
                    display.setCurrent(setupMenu);
                    
                    setupMenu.setCommandListener(new CommandListener()
                    {
                        public void commandAction(Command c, Displayable d)
                        {
                            if (c == back) 
                            {
                                display.setCurrent(menu);
                            }
                            else if (c == List.SELECT_COMMAND)
                            {
                                switch (setupMenu.getSelectedIndex())
                                {
                                    case 0: // resolution
                                        final List resList = new List("Choose Resolution",  Choice.IMPLICIT, new String[]{"80x64","160x128","320x240"}, null);
                                        display.setCurrent(resList);

                                        resList.setCommandListener(new CommandListener()
                                        {
                                            public void commandAction(Command c, Displayable d)
                                            {   
                                                if (c == List.SELECT_COMMAND)
                                                {
                                                    picResolution = resList.getSelectedIndex()+1;
                                                    display.setCurrent(setupMenu);
                                                }
                                            }
                                        });
                                        break;

                                    case 1: // cam node id
                                        final TextBox camnodeBox = new TextBox("Camera ID Node",cameraNodeID,4,0);
                                        final Command ex = new Command("Back", Command.SCREEN, 2);
                                        final Command ok = new Command("OK", Command.SCREEN, 2);
                                        camnodeBox.addCommand(ex);
                                        camnodeBox.addCommand(ok);
                                        display.setCurrent(camnodeBox);

                                        camnodeBox.setCommandListener(new CommandListener()
                                        {
                                            public void commandAction(Command c, Displayable d)
                                            {
                                                if (c == ex)
                                                {
                                                    display.setCurrent(setupMenu);
                                                }
                                                else
                                                if (c == ok)
                                                {
                                                    cameraNodeID = camnodeBox.getString();
                                                    display.setCurrent(setupMenu);
                                                }
                                            }
                                        });
                                        break;
                
                                case 2: // commport
                                    final List commList = new List("Choose Resolution",  Choice.IMPLICIT, conn.getPortStrings(), null);
                                    display.setCurrent(commList);

                                    commList.setCommandListener(new CommandListener()
                                    {
                                        public void commandAction(Command c, Displayable d)
                                        {
                                            if (c == List.SELECT_COMMAND)
                                            {
                                                comPort = commList.getSelectedIndex();
                                                display.setCurrent(setupMenu);
                                            }
                                        }
                                    });
                                    break;
                                }
                            }
                        }
                });
                break;
                
                case 1:

                    // TODO: image item will be appended -> remove before appending again
//                    imageForm.deleteAll();
                    byte imgBuffer[] = getPicture();
                    if (imgBuffer != null)
                    {
                        display.setCurrent(imageForm);
                        final Command imgBack = new Command("Back", Command.SCREEN, 2);
                        Image img = Image.createImage(imgBuffer, 0, imgBuffer.length);
                        imageItem = new ImageItem("ScatterWeb Camera", img, ImageItem.LAYOUT_CENTER, "img");

                        if (imageForm.size() == 1) imageForm.set(0, imageItem);
                        else imageForm.append(imageItem);
                        imageForm.set(0, imageItem);
                        imageForm.addCommand(imgBack);
                        display.setCurrent(imageForm);
                        imageForm.setCommandListener(new CommandListener()
                        {
                            public void commandAction(Command c, Displayable d)
                            {
                                if (c == imgBack) 
                                {
                                    display.setCurrent(menu);
                                }
                            }
                        });
                    }
                    break;
                    
                case 2:
                    break;
                
                case 3:
                    destroyApp(false);
                    notifyDestroyed();
                    break;
            }
        }
            
    }
    
}




