/*
 * RS232Connection.java
 *
 * Created on 3. Dezember 2004, 18:13
 */

package CommTest;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.DataInputStream;
import java.util.Vector;

/**
 *
 * @author  schran
 */
public class RS232Connection 
{
    private CommConnection  conn;
    private InputStream     inputStream;
    private OutputStream    outputStream;
    private Vector          ports;
//    private TextBox         t;
    
    /** Creates a new instance of RS232Connection */
    public RS232Connection() 
    {
//        this.t = t;
        ports = new Vector();
        
        String p = System.getProperty("microedition.commports");
        int comma = p.indexOf(',');
        int start = 0;
        
//        t.setString("Available Commport:"+p);
        
        // only one port found
        if (comma <= 0)
        {
            ports.addElement((String) p);
        }
        else
        // parse all available ports
        while (comma > 0)
        {
//            System.out.println(p.substring(0, comma));
            ports.addElement((String) p.substring(0, comma));
            p = p.substring(comma+1);
            comma = p.indexOf(',');
        }
    }

    /*
     * Return all available ports
     */
    public Vector getPorts()
    {
        return ports;
   }
    
    /*
     * Return all available ports as String array
     */
    public String []getPortStrings()
    {
        String s[] = new String[ports.size()];
        
        for (int i = 0; i < ports.size(); i++) s[i] = (String) ports.elementAt(i);

        return s;
    }
    
    /*
     * Return available port at index i
     */
    public String getPort(int i)
    {
        return (String) ports.elementAt(i);
    }

    public int getBaudRate()
    {
        return conn.getBaudRate();
    }
    
    /*
     * Open the connection
     */
    public boolean open(String port, String baudrate)
    {
        try
        {
            conn = (CommConnection) Connector.open("comm:"+port+";baudrate="+baudrate); 
            //+";bitsperchar=8;parity=none;stopbits=1", Connector.READ_WRITE);
        }
        catch (Exception e)
        {
//            t.setString("Fehler: "+e.getMessage());
            return false;
        }
        return true;
    }
    
    public boolean open(int port, String baudrate)
    {
        return open((String) ports.elementAt(port), baudrate);
    }
    
    /*
     * Closing the connection
     */
    public boolean close()
    {
        try
        {
            conn.close();
        }
        catch (Exception e)
        {
//            t.setString("Fehler: "+e.getMessage());
            return false;
        }
        return true;
    }
    
    /*
     * try to open an input stream
     */
    public boolean openInputStream()
    {
        try
        {
            inputStream = conn.openInputStream();
        }
        catch (Exception e)
        {
//            t.setString("Fehler: "+e.getMessage());
            return false;
        }
        return true;
    }
    
    /*
     * close input stream
     */
    public boolean closeInputStream()
    {
        try
        {
            inputStream.close();
        }
        catch (Exception e)
        {
//            t.setString("Fehler: "+e.getMessage());
            return false;
        }
        return true;
    }

    /*
     * try to open an output stream
     */
    public boolean openOutputStream()
    {
        try
        {
            outputStream = conn.openOutputStream();
        }
        catch (Exception e)
        {
//            t.setString("Fehler: "+e.getMessage());
            return false;
        }
        return true;
    }
    
    /*
     * close output stream
     */
    public boolean closeOutputStream()
    {
        try
        {
            outputStream.close();
        }
        catch (Exception e)
        {
//            t.setString("Fehler: "+e.getMessage());
            return false;
        }
        return true;
    }
    
    public void closeAll()
    /*
     * close all streams and connections
     */
    {
        try
        {
            if (inputStream != null)
            {
                inputStream.close();
            }
            inputStream = null;
            
            if (outputStream != null)
            {
                outputStream.close();
            }
            outputStream = null;

            if (conn != null)
            {
                conn.close();
            }
            conn = null;
        }
        catch (Exception e)
        {
        }
    }

    public int inputAvailable()
    {
        int i = -1;
        try
        {
            i = inputStream.available(); 
        }
        catch (Exception e)
        {
        }
        
        return i;
    }
    
    /*
     * read input
     */
    public int readInt()
    {
        int i = -1;
        try
        {
            i = inputStream.read();
        }
        catch (Exception e)
        {
//            t.setString("Fehler in readInt()");
            return -1;
        }
        return i;
    }

    /*
     * write to output stream
     */
    public void write(int i)
    {
        try
        {
            outputStream.write(i);
            outputStream.flush();
        }
        catch (Exception e)
        {
//            t.setString("Fehler in write()");
        }
    }

    /*
     * send and execute esb terminal command with parameters
     */
    public void execCommand(String command)
    {
        for (int i = 0; i < command.length(); i++)
        {
            int code = (int) command.charAt(i);
            write(code);
        }
        // execute
        write(13);
        write(10);
    }     
}
