/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

#include "ScatterWeb.RSA.h"

struct Sended_Key {
  long int rsa_E;
  long int rsa_N;
};

long int RSA_PUB_N;     // N part of the public key
long int RSA_PUB_E;     // E Part of the public key
long int RSA_PRV_D;     // D Part of the private key
long int RSA_PUB_REC_N; // N Part of the reciever public key
long int RSA_PUB_REC_E; // E Part of the reciever public key
int PUB_KEY_REC_FLG;    // Flag to dtermine when the public key is already recieved

struct Byte_Fragments {
  long int left;
  long int right;
};

struct coded_payload {
  int index;
  struct Byte_Fragments* data;
};

// get the public key
COMMAND(pub, 0){ 
  RSA_getPublicKey(String_parseInt(&str[4], NULL));
  Comm_print("Key request to %i\r\n", String_parseInt(&str[4], NULL)); 
}

int isPrim(int num) {
  int i;
  for(i=3; i<num/2; i++) if (num % i == 0) return 0;
  return 1;
}

// Random Numbers generator
int random(){
  time_t t;
  int acc = 0;
  Time_getSystemTime(&t);
  acc = acc + t.millis ;
  acc = acc % 100 ;
  if(acc % 2 == 0) acc = acc - 1;
  // to give odd number
  while  ((isPrim(acc))== 0 ) {
    System_startWatchdog();
    acc = acc -2 ;
  }
  return acc; 
} 

// Procedure to get the gcd
int ggt(int num1,int num2){
  int tmp1;
  int tmp2;
  int g = 1;
  if(num1 <num2) {
   tmp1 = num2;
   tmp2 = num1;
  }
  else {
    tmp1 = num1;
    tmp2 = num2;
  }
  while(tmp2 > 1) {
    int rest = tmp1 % tmp2;
    if(rest == 0) g = tmp2;
    tmp1 = tmp2;
    tmp2 = rest;
  }
  return g;
}

bool RSA_init(){
  int p, q;
  long int e, m, n;
  long int d = 0;
  long int tmp;
  p = random();
  q = random();
  while(p == q) {
    System_startWatchdog();
    nop();
    q = random();
  }
  n = p * q;
  m = (p-1)*(q-1);
  e = (m-3)/2;
  while(!( (e > 3) && (ggt(m,e) == 1))) {
    System_startWatchdog();
    e = e - 1;
  }
 tmp = 1;
 int i = 1;
 // calculate d
 while(tmp != 0) {
    System_startWatchdog();
    tmp =  (1 +(i*m)) % e;
    if (tmp == 0 ) d = (1 +(i*m)) / e;
    i++;
  }
  RSA_PUB_N = n ;
  RSA_PUB_E = e;
  RSA_PRV_D = d;
  return True;
}

long int getRest(long int base, long int exp, long int n) {
	long int res = 1;
	long int temp;
	long int i = 0;
	if(base==0 || (base==0 && exp==0)) return 0;
	if(exp == 0 || base == 1) return 1;
	while(i < exp) {
	  System_startWatchdog();
    i++;
		temp = res * base;
		if(temp  < n && i < exp) {
			while(temp < n && i<exp) {
			  System_startWatchdog();
				temp = temp*base;
				i++;
			}
		}
		res = temp % n;
		temp = 1;
	}
	return res;
}

// Function to decoding the recieved rsa crypted  token
struct Byte_Fragments rsa_decode(struct Byte_Fragments token) {
  token.left = getRest(token.left, RSA_PRV_D, RSA_PUB_N);
  token.right = getRest(token.right,RSA_PRV_D, RSA_PUB_N);
  return token;
}

// Function to combine the 2 nibbles in one Byte
int get_Byte(struct Byte_Fragments num) {
  int first;
  int tmp;
  tmp = num.left;
  // get the first part
  first = tmp <<4;
  // get second part
  tmp = first + num.right;
  return tmp;
}

packet_t* RSA_encodePayload(packet_t* packet) {
  int i; // the length of the payload
  UINT8 token;
  int first ,second;
  long int left ,right;
  struct Byte_Fragments sk;
  struct Byte_Fragments arra[packet->data_length];
  // loop to iterate all the load bytes
  for(i=0;i<packet->data_length;i++) {
    System_startWatchdog();
    // split data unit in zwei nibble
    token = packet->data[i];
    first = token;
    first = first & 240;
    first = first >>4;
    left = (long int)first;
    // get second part
    second = token & 15;
    right = (long int)second;
    // encoding ....
    left = getRest(left,RSA_PUB_REC_E,RSA_PUB_REC_N);
    right = getRest(right,RSA_PUB_REC_E,RSA_PUB_REC_N);
    // fill in structure 
    sk.left = left;
    sk.right = right;
    // add it to array 
    arra[i] = sk;
  } // end for i
  packet->data = (UINT8*) &arra;
  packet->data_length = sizeof(arra);
  return packet;
}

packet_t* RSA_decodePayload(packet_t* packet) {
  UINT8 token;
  int i;
  struct Byte_Fragments* kst;
  struct Byte_Fragments sst; 
  struct Byte_Fragments ssst; 
  int new_len;
  new_len = packet->data_length / sizeof(sst);
  struct Byte_Fragments* arra;
  arra = (struct Byte_Fragments*)packet->data;
  UINT8 temp[new_len];
  for(i=0;i<new_len;i++) {
    System_startWatchdog();
    kst = arra + i;
    sst = *kst;
    ssst = rsa_decode(sst);
    token = get_Byte(ssst);
    temp[i] = token;
  }
  packet->data_length = new_len;
  packet->data = temp;
  return packet;
}

bool RSA_getPublicKey(int id) {
  packet_t packet;
  PUB_KEY_REC_FLG = False ;
  packet.to = id;             packet.type = RSA_KEYREQUEST;
  packet.header_length = 0;   packet.header = NULL;
  packet.data_length = 0;     packet.data = NULL;
  Net_send(&packet, NULL);
  return True;
}

bool RSA_isPublicKeyReceived() {
  return PUB_KEY_REC_FLG;
}

bool RSA_keyRequestHandler() {
  packet_t packet;
  packet.to = rxPacket.from;
  packet.type = RSA_KEYREPLY;
  packet.header_length = 0;
  packet.header = NULL;
  struct Sended_Key sk;
  sk.rsa_E = RSA_PUB_E;
  sk.rsa_N = RSA_PUB_N;
  packet.data_length = sizeof(sk); //n=4 B and e=4B 
  packet.data = (UINT8*) &sk;
  Net_send(&packet, NULL);
  return True;
}

bool RSA_keyReplyHandler() {
  struct Sended_Key* sk;
  sk = (struct Sended_Key *)rxPacket.data;
  RSA_PUB_REC_N = sk->rsa_N;
  RSA_PUB_REC_E = sk->rsa_E;
  PUB_KEY_REC_FLG = 1;
  Comm_print("Key request from %i\r\n", rxPacket.from); 
  return True;
}
