/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

#include <string.h>
#include "ScatterWeb.SMS.h"

#define CONNECT 1
#define AT_WAIT 2
#define AT_OK 3
#define AT_CPIN_WAIT 4
#define AT_CPIN_OK 5
#define AT_CPIN_XXXX_WAIT 6
#define AT_CPIN_XXXX_ERROR 7
#define AT_CMGD_WAIT 8
#define AT_CPMS_WAIT 9
#define AT_CMGR_WAIT 10

int state;

int pos;

void clean(char *data, int length) {
  int i = 0;
  while(i<length) {
    *(data+i) = 0;
    i++;    
  }
  i = 0;
}

int strlength(char* string) {
   int i = 0;
   for(;*(string+i)!=0;i++);
   return i;
}

UINT8 input[320];  
UINT8 remoteNumber[12];
volatile sms_t rxSMS; // Static struct for received SMS, user functions should use this!

void bit8ToBit7() {
  int i, add1, add2;
  int posOct, posSep; 
  int shift;
  int lenBit7;
  int lenBit8;
  lenBit8 = strlen(input);
  lenBit7 = lenBit8 + ((lenBit8-1)/7) + 1;
  for(i = 0; i < lenBit8; i++) input[i+lenBit7] = input[i];  
  for(i = 0; i < lenBit7; i++) input[i] = '\0';  
  input[lenBit8+lenBit7] = '\0'; 
  for(posOct = lenBit7, posSep = 0, shift = 1; posOct < lenBit7+lenBit8; posOct++,posSep++) {
    add1           =  ((unsigned char)(input[posOct] << shift)) >> 1;
    input[posSep] += add1; 
    add2             = input[posOct] >> (8-shift);
    input[posSep+1] += add2;
    if(shift != 7) shift++;
    else {
      shift = 1;
      posSep++;
    }
  }
  input[lenBit7]='\0';
}  

int hex2Int(unsigned char c) {
  switch(c){
    case 'A': return 10;
    case 'B': return 11;   
    case 'C': return 12;
    case 'D': return 13;
    case 'E': return 14;
    case 'F': return 15;
    default: return (int) (c-48);
  }
}

void hex2Char() {
  int i;
  int len = strlen(input);
  for(i = 0; i < len; i+= 2) {
    input[i/2] = hex2Int(input[i])*16 + hex2Int(input[i+1]);
  }
  input[len/2]='\0';
}

void movePdu() {
  int start;
  int end = strlen(input);
  int i;
  while(end > 0) {
    if( input[end-1] == '\r' && ((input[end-2] > 47 && input[end-2] < 58) || (input[end-2] > 64 && input[end-2] < 71))) {
      end--;
      break;
    }
    else end--;
  }
  start = end;
  while(start > 0){
    if(input[start-1] == '\n') break;
    else start--;
  }
  for(i = start; i < end; i++) input[i-start] = input[i];
  input[i-start] = '\0';
}

bool receiveATComment(char *answer) {
  if(strstr(input, answer)) return 1;
  else return 0;
}

int findUserDataLength(unsigned char * sms) {
	int pos =0;
	int TP_UDH_exist = 0;
	int offset;
	offset = hex2Int(sms[pos])*16 + hex2Int(sms[pos+1])*2;
	pos    += 2;
	pos    += offset;
	if(((sms[pos]-48)>>2) == 1) TP_UDH_exist = 1;
	else TP_UDH_exist = 0;
	pos += 2;				 // TP_FO
	offset = hex2Int(sms[pos])*16 + hex2Int(sms[pos+1]);
	pos += 2;
	pos += 2;
	pos += offset;
	pos += 2;				 // TP_PID
	pos += 2;				 // TP_DCS
	pos += 14;			 // TP_SCTS
	pos += 2;				 // TP_UDL
	if(TP_UDH_exist) {
		offset = hex2Int(sms[pos])*16 + hex2Int(sms[pos+1])*2;
		pos += 2;			 // TP_UDH_Length
		pos += offset; // TP_UDH
	}
	return pos;
}

void moveToText() {
  int i;
  int len = findUserDataLength(input);
  for(i = len; i < strlen(input); i++) input[i-len] = input[i];
  input[i-len] = '\0';
}

void setState(int i) { 
  state = i;
}

fp_t messageCallback;
fp_vp numberCallback;

void numberSender() {
  if(numberCallback) numberCallback((void*)&SMS_numberOfSms);
}

void getNumberOfSms() {
  SMS_numberOfSms = (*(strstr(input, "+CPMS: ") + 7)) - 48;
  if((*(strstr(input, "+CPMS: ") + 8)) != ',') SMS_numberOfSms = 10*(SMS_numberOfSms)+ ((*(strstr(input, "+CPMS: ") + 8)) - 48);
  Timers_add(1000, numberSender, 0xFFFF);
}

void cleanInput(int *pos) {
  clean(input, strlength(input));
  *pos = 0;
}

bool SMS_registerMessageCallback(fp_t fp) {
  messageCallback = fp;
  return True;
}

bool SMS_registerNumberCallback(fp_t fp) {
  numberCallback = fp;
  return True;
}

void get(UINT8* number) {
  rxSMS.message = input;
  rxSMS.number = number;
  if(messageCallback) messageCallback((void*)&rxSMS);
}

void sendATComment(char *AT_Comment) { 
  Comm_print("%s\r\n", AT_Comment);
}

void isMINRequired() {
  sendATComment("AT+CPIN?");
  setState(AT_CPIN_WAIT);
}

void getRemoteNumber() {
	int pos =0;
	int TP_UDH_exist = 0;
	int offset;
	int i = 0;
	clean(remoteNumber,strlength(remoteNumber));
	offset = hex2Int(input[pos])*16 + hex2Int(input[pos+1])*2;
	pos    += 2;
	pos    += offset;
	if(((input[pos]-48)>>2) == 1) TP_UDH_exist = 1;
	else 	TP_UDH_exist = 0;
	pos += 2; // TP_FO
	offset = hex2Int(input[pos])*16 + hex2Int(input[pos+1]);
	pos += 2;
	pos += 2;
	remoteNumber[i]='0';
	for(i=2; i<offset;i++){
    if(i%2) remoteNumber[i-2]= input[pos+i];
    else remoteNumber[i]= input[pos+i];
  }
  remoteNumber[i-1]= '\0';
}

char sms_min[4];

void setMIN() {
  Comm_print("AT+CPIN=");
  Comm_print("%c",sms_min[0]);
  Comm_print("%c",sms_min[1]);
  Comm_print("%c",sms_min[2]);
  Comm_print("%c",sms_min[3]);
  Comm_print("\r\n");
  setState(AT_CPIN_XXXX_WAIT);
}

void smsRx(UINT8 temp2) {
  UINT8 temp = temp2;
  if( ((state == CONNECT) && ((input[0]=='\0' && temp == 'A') || input[0]== 'A')) || state != CONNECT) {
    input[pos] = temp;
    if( (input[pos-1] == 'O' && input[pos] =='K') ||
      (input[pos-4] == 'E' && input[pos-3] =='R' && input[pos-2] =='R' && input[pos-1] =='O' && input[pos] =='R')) {
      input[pos+1] = '\0'; 
      switch(state) {
        case AT_WAIT: {
          if(receiveATComment("OK")) {
            setState(AT_OK);
            cleanInput(&pos);   
            Timers_add(50, isMINRequired, 0xFFFF);
          }
          else cleanInput(&pos);
        break;
        } 
        case AT_CPIN_WAIT: {
          if(receiveATComment("+CPIN: READY")) {
            setState(CONNECT);
            cleanInput(&pos);
            break;
          }
          if(receiveATComment("+CPIN: SIM PIN")) {
            setState(AT_CPIN_XXXX_WAIT);
            cleanInput(&pos);  
            Timers_add(50, setMIN, 0xFFFF);
            break;
          }
        break;
        } 
        case AT_CPIN_XXXX_WAIT: { 
          if(receiveATComment("OK")) {
            setState(CONNECT);
            cleanInput(&pos);
            break;
          }
          if(receiveATComment("ERROR")) {
            setState(AT_CPIN_XXXX_ERROR);
            cleanInput(&pos);  
            break;
          }
        break;
        } 
        case  AT_CPMS_WAIT: {
          if(receiveATComment("+CPMS: ")) {
            getNumberOfSms();
            setState(CONNECT);
          }
          cleanInput(&pos);
        break;
        } 
        case  AT_CMGD_WAIT: {
          if(receiveATComment("OK")) setState(CONNECT);
          cleanInput(&pos);
        break;
        } 
        case  AT_CMGR_WAIT: {
          if(receiveATComment("+CMGR: ")) {
            setState(CONNECT);
            movePdu();
            getRemoteNumber();
            moveToText();
            hex2Char();
            bit8ToBit7();
            get(remoteNumber);
          }
          cleanInput(&pos);
        break;
        } 
        default: break;
      }
    }
    else pos++;
  }
}

bool SMS_init(char min[4]) {
  Comm_init(BPS_19200);
  sms_min[0] = min[0];
  sms_min[1] = min[1];
  sms_min[2] = min[2];
  sms_min[3] = min[3];
  SMS_registerMessageCallback(0);
  SMS_registerNumberCallback(0);
  Comm_registerByteLevel(smsRx);
  Comm_clear();
  pos = 0;
  SMS_numberOfSms = 0;
  clean(input, strlength(input));
  clean(input,strlength(remoteNumber));
  setState(AT_WAIT);
  sendATComment("AT");
  return True;
}

bool SMS_isMessageAvailable() {
  if(state != CONNECT) return False;
  sendATComment("AT+CPMS=SM");
  setState(AT_CPMS_WAIT);
  return True;
}

bool SMS_deleteMessage(int number) {
  Comm_print("AT+CMGD=%i\r\n", number);
  setState(AT_CMGD_WAIT);
  return True;
}

bool SMS_getMessage(int number) {
  if(state != CONNECT) return False;
  Comm_print("AT+CMGR=%i\r\n", number);
  setState(AT_CMGR_WAIT);
  return True;
}

char getChar(char c) {
   	switch(c){
    	case 0x00: 
    		return '0';
    	case 0x01:
    		return '1';
    	case 0x02:
    		return '2';
    	case 0x03:
    		return '3';
    	case 0x04:
    		return '4';
    	case 0x05:
    		return '5';
    	case 0x06:
    		return '6';
    	case 0x07:
    		return '7';
    	case 0x08:
    		return '8';
    	case 0x09:
    		return '9';
    	case 0x0A:
    		return 'A';
    	case 0x0B:
    		return 'B';
    	case 0x0C:
    		return 'C';
    	case 0x0D:
    		return 'D';
    	case 0x0E:
    		return 'E';
    	case 0x0F:
    		return 'F';
    }
    return 0;
}

char getGSM(char c) {
	switch(c) {
		case '@': 
			return 0x00;
		case '$':
			return 0x02;
		case '\n':
			return 0x0A;
		case '\r':
			return 0x0D;
		case '_':
			return 0x11;
		case ' ':
			return 0x20;
		case '!':
			return 0x21;
		case '"':
			return 0x22;
		case '#':
			return 0x23;
		case '%':
			return 0x25;
		case '&':
			return 0x26;
		case '\'':
			return 0x27;
		case '(':
			return 0x28;
		case ')':
			return 0x29;
		case '*':
			return 0x2A;
		case '+':
			return 0x2B;
		case ',':
			return 0x2C;
		case '-':
			return 0x2D;
		case '.':
			return 0x2E;
		default:
			return c;
	}
	return c;
}

bool SMS_sendMessage(char* number, char* text) {
  if(state != CONNECT) return False;
  unsigned char msg[320];
  unsigned char pdutext[161];
  int i = 0;
  int length= 0;
  int x = 0;
  int count = 0;
  unsigned char temp;
  unsigned char temp2;
  length = strlength(number);
  clean(msg, 320);
  clean(pdutext, 161);
  msg[0] = '0';
  msg[1] = '0';
  msg[2] = '1';
  msg[3] = '1';
  msg[4] = '0';
  msg[5] = '0';
  msg[6] = '0';
  msg[7] = 'B';
  msg[8] = '9';
  msg[9] = '1';
  i = 10;
  for(;x<length/2;x++){
  	msg[i+(2*x)+1] = *(number+(2*x));
  	msg[i+(2*x)] = *(number+(2*x)+1);
  }
  if(length%2 != 0){
  	msg[i+(2*x)+1] = *(number+(2*x));
  	msg[i+(2*x)] = 'F'; 
  	i = i+(2*x)+2;	
  } else i = i+(2*x);
  x = 0;
  length = strlength(text);
  msg[i] = '0';
  msg[i+1] = '0';
  msg[i+2] = '0';
  msg[i+3] = '0';
  msg[i+4] = 'A';
  msg[i+5] = 'A';
  msg[i+6] = getChar(((((unsigned char)length) & 0xF0)>>4));
  msg[i+7] = getChar((((unsigned char)length) & 0x0F));
  length = strlength(text);
  if(length>160) return False;
  for(;x<length;x+=8) {
    if(x+1<=length) {
      temp = getGSM(*(text+x+1)) & 0x01;
      temp <<=7;
      pdutext[x-count] = getGSM(*(text+x)) | temp; 
    }
    if(x+2<=length) {
      temp = getGSM(*(text+x+2)) & 0x03;
      temp <<=6;
      temp2 = getGSM(*(text+x+1));
      temp2 >>=1;
      pdutext[x+1-count] = temp2 | temp;
    }
    if(x+3<=length) {
      temp = getGSM(*(text+x+3)) & 0x07;
      temp <<=5;
      temp2 = getGSM(*(text+x+2));
      temp2 >>=2;
      pdutext[x+2-count] = temp2 | temp; 
    }
    if(x+4<=length) {
      temp = getGSM(*(text+x+4)) & 0x0F;
      temp <<=4;
      temp2 = getGSM(*(text+x+3));
      temp2 >>=3;
      pdutext[x+3-count] = temp2 | temp;
    }
    if(x+5<=length) {
      temp = getGSM(*(text+x+5)) & 0x1F;
      temp <<=3;
      temp2 = getGSM(*(text+x+4));
      temp2 >>=4;
      pdutext[x+4-count] = temp2 | temp; 
    }
    if(x+6<=length) {
      temp = getGSM(*(text+x+6)) & 0x3F;
      temp <<=2;
      temp2 = getGSM(*(text+x+5));
      temp2 >>=5;
      pdutext[x+5-count] = temp2 | temp; 
    }
    if(x+7<=length) {
      temp = getGSM(*(text+x+7)) & 0x7F;
      temp <<=1;
      temp2 = getGSM(*(text+x+6));
      temp2 >>=6;
      pdutext[x+6-count] = temp2 | temp;
    }
    count++;
  }
  x = 0;
  length = strlength(pdutext);
  for(;x<length;x++) {
  	temp = *(pdutext+x);
  	temp = temp & 0xF0;
  	temp >>= 4;
  	msg[i+8+(2*x)] = getChar(temp);
  	temp = *(pdutext+x);
  	temp = temp & 0x0F;
  	msg[i+8+(2*x)+1] = getChar(temp);
  }
  msg[i+8+(2*x)]=0x1A;
  length = (strlength(msg)-3)/2;
  Comm_print("AT+CMGF=0\r\n");
  System_wait(50000);
  Comm_print("AT+CMGS=%i\r\n", length);
  System_wait(20000);
  Comm_print(msg); 
  Comm_print((UINT8*)'\26');
  return True;
}

bool SMS_isConnected() {
  if(state == AT_WAIT) return False;
  if(state == AT_OK) return False;
  if(state == AT_CPIN_WAIT) return False;
  if(state == AT_CPIN_XXXX_WAIT) return False;
  if(state == AT_CPIN_XXXX_ERROR) return False;
  return True;
}
