/*
Copyright 2003/2004, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.
Contributors: Bjoern Lichtblau

For documentation and questions please use the web site
http://www.scatterweb.net and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2003/2004/2005
*/

/** \file ScatterWeb.Routing.c
 ** \ingroup Applications
 ** \brief A simple tree routing for the Scatterweb.
 ** <p>
 ** This module implements a simple tree rouing algorithm for the Scatterweb.
 ** It needs one root in the sensor net (usually the Stick) and every sensor
 ** tries to find a route to the root by attaching to a sensor who already
 ** knows how to route to the root.
 ** <p>
 ** If the network is established you can use ::routing_rootcast to send
 ** data to the root or routing_treecast to send packets down the whole
 ** tree.  
 ** <p>
 ** Each node periodically braodacsts its status in a ROUTING_INFO packet
 ** and every node has a routing table of all its heard neighbors (see Info packets).
 ** Neighbors which are silent for a long time age out (see Aging).
 ** If a node has no parent (and so no path to the root) it tries to
 ** find one in the routing table and requests to become a child (see Requesting 
 ** Accept & Deny ).     
 ** <p>
 ** The routing_rootcast works by repeatedly sending the packet to the parent until
 ** the root is found.
 ** routing_treecast works by repeatedly sending a packet to all known children.     
 **   
 ** @author        Marco Hutta
 ** @author        Bjoern Lichtblau
 ** @version       2.2
 ** @since         2.1
 **/ 

#ifndef ScatterWeb_Routing
#define ScatterWeb_Routing

#include "../../../../../System/src/ScatterWeb.System.h"

#define AS_GATE            (0x0000)
#define AS_NODE            (0xFFFF)
#define ROUTING_TABLE_SIZE (20)
#define RPT_ROOTINFO       (0x0001)
#define RPT_SENSOR         (0x0002)
#define RPT_TEXT           (0x0003)
#define RPT_TERM_REPLY     (0x0004)
#define RPT_NEWCHILD       (0x0005)
#define RPT_LOSTCHILD      (0x0006)
#define TPT_TERM_REQUEST   (0x0001)

bool Routing_init(UINT16 mode);
bool Routing_infoHandler();
bool Routing_requestHandler();
bool Routing_acceptHandler();
bool Routing_denyHandler();
bool Routing_noChildHandler();
bool Routing_rootPacketHandler();
bool Routing_treePacketHandler();
char Routing_rootCast(UINT16 type, UINT8* data, UINT16 length);
char Routing_treeCast(UINT16 to, UINT16 type, UINT8* data, UINT16 length);

#endif
