/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

#include "ScatterWeb.Event.h"
#include "ScatterWeb.Neighbours.h"
#include "lib/ScatterWeb.Routing.h"
#include "../../../../System/src/ScatterWeb.System.h"

void Process_interceptRadioHandler() {

}

void Process_radioHandler() {
  if ( !(rxPacket.to == BROADCAST || rxPacket.to == Configuration.id) ) {
    Process_interceptRadioHandler();
  }
  else {
    switch(rxPacket.type){
      case ACK_PACKET: break;
      case MESSAGING_REQ_PACKET: Messaging_requestHandler(); break;
      case MESSAGING_ANS_PACKET: Messaging_answerHandler(); break;
      case ROUTING_INFO: Routing_infoHandler(); break;
      case ROUTING_REQUEST: Routing_requestHandler(); break;
      case ROUTING_ACCEPT: Routing_acceptHandler(); break;
      case ROUTING_DENY: Routing_denyHandler(); break;
      case ROUTING_NOCHILD: Routing_noChildHandler(); break;        
      case ROUTING_ROOTPACKET: Routing_rootPacketHandler(); break;
      case ROUTING_TREEPACKET: Routing_treePacketHandler(); break;
      case VGATE_REPLY: Neighbours_replyHandler(); break;
      case PING_PACKET: 
         if (rxPacket.to == Configuration.id) {
           packet_t packet;
           packet.to = rxPacket.from;    packet.type = PONG_PACKET;
           packet.header_length = 0;     packet.header = NULL;
           packet.data_length = 0;       packet.data = NULL;
           Net_send(&packet, NULL);
         }
         break;
      case PONG_PACKET: 
          if (rxPacket.to == Configuration.id) {
            Comm_print("PONG from %i\r\n", rxPacket.from);   
          }
          break;
      default: break;
    }
  }
}

COMMAND(app, 0) {
  String_write(term_reply, "vGate");
}

void Process_init(){
  System_registerCallback(C_RADIO, Process_radioHandler); 
  Event_init();
  Routing_init(AS_NODE);
  Data_redOn();
  Data_beeperOn();
  System_wait(1000);
  Data_beeperOff();
  Data_redOff();
  Comm_log(LOW, "| Application %s initialized\r\n\r\n", imageData.versionName);
}

// Software header. See programming.h for description.
// Theses variables are defined by the linker script, they are needed in the software header
extern int __image_start;
extern int __image_end;
extern UINT16 prog_reprogram();
IMAGE_DATA const imageData_t imageData = {
    (UINT16)&__image_start, (UINT16)&__image_end,

    6, // ECR
    (UINT16)prog_reprogram,
    "ECR: V2.30", // Must be 20 chars max (with \0 21 alltogether).
    __DATE__ " " __TIME__        // Only filled when programming.o is rebuilt!
};
