/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

/** \file ScatterWeb.RFC.h
 ** \ingroup Applications
 ** \brief Methods for handling Remote Function Calling (RFC).
 **
 ** @author        Marco Hutta
 ** @version       2.2
 ** @since         2.2
 **/ 
    
#ifndef ScatterWeb_RFC
#define ScatterWeb_RFC

#include "../../../../System/src/ScatterWeb.System.h"

// Functions of ScatterWeb.RFC
#define RFC_INIT                         0

// Functions of ScatterWeb.Comm
#define COMM_PRINT                       1
#define COMM_LOG                         2
#define COMM_ON                          3
#define COMM_OFF                         4
#define COMM_REGISTERBYTELEVEL           5
#define COMM_CLEAR                       6

// Functions of ScatterWeb.Configuration
#define CONFIGURATION_INIT               7
#define CONFIGURATION_SAVE               8
#define CONFIGURATION_PRINT              9

// Functions of ScatterWeb.Data
#define DATA_INIT                        10
#define DATA_BATTERYON                   11
#define DATA_BATTERYOFF                  12
#define DATA_BEEPERON                    13
#define DATA_BEEPEROFF                   14
#define DATA_BEEPERTOGGLE                15
#define DATA_BUTTONON                    16
#define DATA_BUTTONOFF                   17
#define DATA_GREENON                     18
#define DATA_GREENOFF                    19
#define DATA_GREENTOGGLE                 20
#define DATA_REDON                       21
#define DATA_REDOFF                      22
#define DATA_REDTOGGLE                   23
#define DATA_YELLOWON                    24
#define DATA_YELLOWOFF                   25
#define DATA_YELLOWTOGGLE                26
#define DATA_MICROPHONEON                27
#define DATA_MICROPHONEOFF               28
#define DATA_MOVEMENTON                  29
#define DATA_MOVEMENTOFF                 30
#define DATA_TEMPERATUREON               31
#define DATA_TEMPERATUREOFF              32
#define DATA_VIBRATIONON                 33
#define DATA_VIBRATIONOFF                34
#define DATA_GREENLED                    35
#define DATA_REDLED                      36
#define DATA_YELLOWLED                   37
#define DATA_RC5SENDPHI                  38
#define DATA_RC5ENABLE                   39
#define DATA_RC5DISABLE                  40
#define DATA_RC5SENDNEC                  41
#define DATA_SENSORHANDLER               42
#define DATA_RC5RECEIVEHANDLER           43
#define DATA_RC5EVENTHANDLER             44

// Functions of ScatterWeb.IO
#define IO_ERASE                         45
#define IO_READ                          46
#define IO_WRITE                         47

// Functions of ScatterWeb.Messaging
#define MESSAGING_ON                     48
#define MESSAGING_SERIALHANDLER          49
#define MESSAGING_REQUESTHANDLER         50
#define MESSAGING_ANSWERHANDLER          51
#define MESSAGING_REQUESTDATAHANDLER     52

// Functions of ScatterWeb.Net
#define NET_INIT                         53
#define NET_ON                           54
#define NET_OFF                          55
#define NET_SETTXPOWER                   56
#define NET_SEND                         57
#define NET_RXHANDLER                    58
#define NET_TXHANDLER                    59
#define NET_ESEND                        60

// Functions of ScatterWeb.String
#define STRING_WRITEUINT                 61
#define STRING_WRITEULONGINT             62
#define STRING_FROMBYTETOHEX             63
#define STRING_FROMHEXTOBYTE             64
#define STRING_WRITETEMPERATURE          65
#define STRING_ABSTRACTWRITE             66
#define STRING_WRITE                     67
#define STRING_PARSEINT                  68
#define STRING_PARSELONGINT              69

// Functions of ScatterWeb.Synchronisation
#define SYNCHRONISATION_INIT             70
#define SYNCHRONISATION_ISROOT           71
#define SYNCHRONISATION_REGISTERSYNCDONE 72
#define SYNCHRONISATION_DOSYNC           73

// Functions of ScatterWeb.System
#define SYSTEM_REGISTERCALLBACK          74
#define SYSTEM_GETTEMPERATURE            75
#define SYSTEM_SETDCO                    76
#define SYSTEM_STARTWATCHDOG             77
#define SYSTEM_STOPWATCHDOG              78
#define SYSTEM_RESET                     79
#define SYSTEM_RESETREASON               80
#define SYSTEM_PANIC                     81
#define SYSTEM_PANICREASON               82
#define SYSTEM_PANICCLEAR                83
#define SYSTEM_WAIT                      84
#define SYSTEM_WAITA                     85
#define SYSTEM_POWERON                   86
#define SYSTEM_POWEROFF                  87
#define SYSTEM_REPROGRAMM                88
#define SYSTEM_STARTCONVERSION           89

// Functions of ScatterWeb.Threading
#define THREADING_INIT                   90
#define THREADING_ADD                    91
#define THREADING_DOEVENTS               92
#define THREADING_SLEEP                  93
#define THREADING_WAIT                   94
#define THREADING_EVENTHANDLER           95
#define THREADING_PACKETHANDLER          96
#define THREADING_SENTPACKETHANDLER      97
#define THREADING_RECV                   98
#define THREADING_RECVP                  99
#define THREADING_SEND                   100
#define THREADING_ESEND                  101

// Functions of ScatterWeb.Time
#define TIME_GETSYSTEMTIME               102
#define TIME_SETSYSTEMTIME               103
#define TIME_CONVERTSYSTEMTIME           104
#define TIME_CONVERTTIME                 105
#define TIME_WRITETIME                   106
#define TIME_READTIME                    107

// Functions of ScatterWeb.Timers
#define TIMERS_INIT                      108
#define TIMERS_ADD                       109
#define TIMERS_RESET                     110
#define TIMERS_REMOVE                    111
#define TIMERS_EVENTHANDLER              112

// Functions for Functions with packets
#define PACKET_HEADER                    113
#define PAKCET_DATA                      114

#endif
