/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

#include "ScatterWeb.RVC.h"

UINT16 RVCID;

void RVC_doUINT(UINT8 i) {
  if(RVCID < 10) String_write(term_reply, "rvc 00%i %i", RVCID, i);
  else if(RVCID < 100) String_write(term_reply, "rvc 0%i %i", RVCID, i);
  else String_write(term_reply, "rvc %i %i", RVCID, i);
}

void RVC_doUINTPointer(UINT8* s) {
  if(RVCID < 10) String_write(term_reply, "rvc 00%i %s", RVCID, s);
  else if(RVCID < 100) String_write(term_reply, "rvc 0%i %s", RVCID, s);
  else String_write(term_reply, "rvc %i %s", RVCID, s);
  
}

void RVC_doCallbacks() {
  if(RVCID < 10) String_write(term_reply, "rvc 00%i %i %i %i", RVCID, callbacks[0], callbacks[1], callbacks[2]);
  else if(RVCID < 100) String_write(term_reply, "rvc 0%i %i %i %i", RVCID, callbacks[0], callbacks[1], callbacks[2]);
  else String_write(term_reply, "rvc %i %i %i %i", RVCID, callbacks[0], callbacks[1], callbacks[2]);
}

COMMAND(rvc, 0) {
  RVCID = String_parseInt(&str[4], NULL);
  switch(RVCID) {
      case COMM_LOGLEVEL: {
        if(String_parseInt(&str[8], NULL) == 1) Comm_logLevel = String_parseInt(&str[10], NULL);
        RVC_doUINT(Comm_logLevel);
      }
      break;
      case CONFIGURATION_CONFIGURATION_ID: {
        if(String_parseInt(&str[8], NULL) == 1) {
          Configuration.id = String_parseInt(&str[10], NULL);
          Configuration_save();
        }
        RVC_doUINT(Configuration.id);
      }
      break;
      case CONFIGURATION_CONFIGURATION_RXRECEIVELIMIT: {
        if(String_parseInt(&str[8], NULL) == 1) {
          Configuration.rxReceiveLimit = String_parseInt(&str[10], NULL);
          Configuration_save();
        }
        RVC_doUINT(Configuration.rxReceiveLimit);
      }
      break;
      case CONFIGURATION_CONFIGURATION_TRANSMITPOWER: {
        if(String_parseInt(&str[8], NULL) == 1) {
          Configuration.transmitPower = String_parseInt(&str[10], NULL);
          Configuration_save();
        }
        RVC_doUINT(Configuration.transmitPower);
      }
      break;
      case CONFIGURATION_CONFIGURATION_BOOT_LOGLEVEL: {
        if(String_parseInt(&str[8], NULL) == 1) {
          Configuration.boot_logLevel = String_parseInt(&str[10], NULL);
          Configuration_save();
        }
        RVC_doUINT(Configuration.boot_logLevel);
      }
      break;
      case CONFIGURATION_CONFIGURATION_PANICFLAGS: {
        if(String_parseInt(&str[8], NULL) == 1) {
          Configuration.panicFlags = String_parseInt(&str[10], NULL);
          Configuration_save();
        }
        RVC_doUINT(Configuration.panicFlags);
      }
      break;
      case CONFIGURATION_CONFIGURATION_FIRMWAREFLAGS: {
        if(String_parseInt(&str[8], NULL) == 1) {
          Configuration.firmwareFlags = String_parseInt(&str[10], NULL);
          Configuration_save();
        }
        RVC_doUINT(Configuration.firmwareFlags);
      }
      break;
      case DATA_SENSORFLAGS: {
        if(String_parseInt(&str[8], NULL) == 1) sensorFlags = String_parseInt(&str[10], NULL);
        RVC_doUINT(sensorFlags);
      }
      break;
      case MESSAGING_TERM_REPLY: {
        if(String_parseInt(&str[8], NULL) == 1) term_reply[String_parseInt(&str[10], NULL)] = str[15];
        RVC_doUINTPointer(term_reply);
      }
      break;
      case NET_RXPACKET_TO: {
        if(String_parseInt(&str[8], NULL) == 1) rxPacket.to = String_parseInt(&str[10], NULL);
        RVC_doUINT(rxPacket.to);
      }
      break;
      case NET_RXPACKET_FROM: {
        if(String_parseInt(&str[8], NULL) == 1) rxPacket.from = String_parseInt(&str[10], NULL);
        RVC_doUINT(rxPacket.from);
      }
      break;
      case NET_RXPACKET_TYPE: {
        if(String_parseInt(&str[8], NULL) == 1) rxPacket.type = String_parseInt(&str[10], NULL);
        RVC_doUINT(rxPacket.type);
      }
      break;
      case NET_RXPACKET_NUM: {
        if(String_parseInt(&str[8], NULL) == 1) rxPacket.num = String_parseInt(&str[10], NULL);
        RVC_doUINT(rxPacket.num);
      }
      break;
      case NET_RXPACKET_HEADER: {
        if(String_parseInt(&str[8], NULL) == 1) rxPacket.header[String_parseInt(&str[10], NULL)] = str[15];
        RVC_doUINTPointer(rxPacket.header);
      }
      break;
      case NET_RXPACKET_HERDER_LENGTH: {
        if(String_parseInt(&str[8], NULL) == 1) rxPacket.header_length = String_parseInt(&str[10], NULL);
        RVC_doUINT(rxPacket.header_length);
      }
      break;
      case NET_RXPACKET_DATA: {
        if(String_parseInt(&str[8], NULL) == 1) rxPacket.data[String_parseInt(&str[10], NULL)] = str[15];
        RVC_doUINTPointer(rxPacket.data);
      }
      break;
      case NET_RXPACKET_DATA_LENGTH: {
        if(String_parseInt(&str[8], NULL) == 1) rxPacket.data_length = String_parseInt(&str[10], NULL);
        RVC_doUINT(rxPacket.data_length);
      }
      break;
      case NET_SENTPACKET_TO: {
        if(String_parseInt(&str[8], NULL) == 1) sentPacket.to = String_parseInt(&str[10], NULL);
        RVC_doUINT(sentPacket.to);
      }
      break;
      case NET_SENTPACKET_FROM: {
        if(String_parseInt(&str[8], NULL) == 1) sentPacket.from = String_parseInt(&str[10], NULL);
        RVC_doUINT(sentPacket.from);
      }
      break;
      case NET_SENTPACKET_TYPE: {
        if(String_parseInt(&str[8], NULL) == 1) sentPacket.type = String_parseInt(&str[10], NULL);
        RVC_doUINT(sentPacket.type);
      }
      break;
      case NET_SENTPACKET_NUM: {
        if(String_parseInt(&str[8], NULL) == 1) sentPacket.num = String_parseInt(&str[10], NULL);
        RVC_doUINT(sentPacket.num);
      }
      break;
      case NET_SENTPACKET_HEADER: {
        if(String_parseInt(&str[8], NULL) == 1) sentPacket.header[String_parseInt(&str[10], NULL)] = str[15];
        RVC_doUINTPointer(sentPacket.header);
      }
      break;
      case NET_SENTPACKET_HEADER_LENGTH: {
        if(String_parseInt(&str[8], NULL) == 1) sentPacket.header_length = String_parseInt(&str[10], NULL);
        RVC_doUINT(sentPacket.header_length);
      }
      break;
      case NET_SENTPACKET_DATA: {
        if(String_parseInt(&str[8], NULL) == 1) sentPacket.data[String_parseInt(&str[10], NULL)] = str[15];
        RVC_doUINTPointer(sentPacket.data);
      }
      break;
      case NET_SENTPACKET_DATA_LENGTH: {
        if(String_parseInt(&str[8], NULL) == 1) sentPacket.data_length = String_parseInt(&str[10], NULL);
        RVC_doUINT(sentPacket.data_length);
      }
      break;
      case NET_TXRETRIES: {
        if(String_parseInt(&str[8], NULL) == 1) txRetries = String_parseInt(&str[10], NULL);
        RVC_doUINT(txRetries);
      }
      break;
      case NET_CDCOUNTER: {
        if(String_parseInt(&str[8], NULL) == 1) cdCounter = String_parseInt(&str[10], NULL);
        RVC_doUINT(cdCounter);
      }
      break;
      case NET_TXSTATE: {
        if(String_parseInt(&str[8], NULL) == 1) txState = String_parseInt(&str[10], NULL);
        RVC_doUINT(txState);
      }
      break;
      case SYNCHRONISATION_SYNCSTRUCT_SEQNUM: {
        if(String_parseInt(&str[8], NULL) == 1) SyncStruct.seqNum = String_parseInt(&str[10], NULL);
        RVC_doUINT(SyncStruct.seqNum);
      }
      break;
      case SYNCHRONISATION_SYNCSTRUCT_MASTERROOTID: {
        if(String_parseInt(&str[8], NULL) == 1) SyncStruct.masterRootId = String_parseInt(&str[10], NULL);
        RVC_doUINT(SyncStruct.masterRootId);
      }
      break;
      case SYNCHRONISATION_SYNCSTRUCT_SYNCPERIODS: {
        if(String_parseInt(&str[8], NULL) == 1) SyncStruct.syncPeriods = String_parseInt(&str[10], NULL);
        RVC_doUINT(SyncStruct.syncPeriods);
      }
      break;
      case SYSTEM_CALLBACKS: {
        if(String_parseInt(&str[8], NULL) == 1) callbacks[String_parseInt(&str[10], NULL)] == (fp_t) String_parseInt(&str[15], NULL);
        RVC_doCallbacks();
      }
      break;
      case SYSTEM_RUNMODULE: {
        if(String_parseInt(&str[8], NULL) == 1) runModule = String_parseInt(&str[10], NULL);
        RVC_doUINT(runModule);
      }
      break;
      case SYSTEM_RXPTEMP: {
        if(String_parseInt(&str[8], NULL) == 1) rxpTemp = String_parseInt(&str[10], NULL);
        RVC_doUINT(rxpTemp);
      }
      break;
      case SYSTEM_RXPVALUE: {
        if(String_parseInt(&str[8], NULL) == 1) rxpValue = String_parseInt(&str[10], NULL);
        RVC_doUINT(rxpValue);
      }
      break;
      case SYSTEM_MIC: {
        if(String_parseInt(&str[8], NULL) == 1) MIC = String_parseInt(&str[10], NULL);
        RVC_doUINT(MIC);
      }
      break;
      case THREADING_THREADING_CURRENTTHREAD: {
        if(String_parseInt(&str[8], NULL) == 1) Threading_currentThread = String_parseInt(&str[10], NULL);
        RVC_doUINT(Threading_currentThread);
      }
      break;
      default: String_write(term_reply, "rvc error"); break;
  }
  Comm_clear();
}
