/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

#include "ScatterWeb.Event.h"

// Base address for application config, if needed.
#define EADR_APPCONFIG  (0x0D00)

// Application config held in ram, in EEPROM at address 0x0000.
appconfig_t AppConfig;

// Loads application config from EEPROM[0x0000]-EEPROM[sizeof(appconfig_t)] to AppConfig.
void appconfig_load(){ IO_read( EADR_APPCONFIG, (UINT8*) &AppConfig, sizeof(appconfig_t)); }

// Saves application config AppConfig to EEPROM[0x0000]-EEPROM[sizeof(appconfig_t)].
void appconfig_save(){ IO_write( EADR_APPCONFIG, (UINT8*) &AppConfig, sizeof(appconfig_t)); }

// Sensor event handler.
void handleEvent(sdata_t* data){
  packet_t p;
  p.to = BROADCAST;
  p.type = SENSOR_PACKET;
  p.header_length = 0;
  p.data = (UINT8*) data;
  p.data_length = sizeof(*data);
  Net_send(&p, NULL);
}

// Configuration commands.
// Enable sensor reporting, which actually does a saf 14 what means take local events and broadcast them via radio.
COMMAND(esr, 0){ 
  String_write(term_reply, "NOT SUPPORTED: %s (wrong application)", str);
}                
  
// Disable sensor reporting, which clears bits 0x14 of announceFlags.
COMMAND(dsr, 0){ 
  String_write(term_reply, "NOT SUPPORTED: %s (wrong application)", str);
}                

// raf <br> Reads announcing state of the sensor board (appconfig_t.announceFlags).
COMMAND(raf, 0){ 
 String_write(term_reply, "NOT SUPPORTED: %s (wrong application)", str);
}                 

// saf xx <br> Writes announcing state of the sensor board (appconfig_t.announceFlags) as a possible combination of AF_SERIAL, AF_RADIO.
COMMAND(saf, 0){ 
  String_write(term_reply, "NOT SUPPORTED: %s (wrong application)", str);
} 

// rsm <br> Read sensor mask (appconfig_t.sensorMask).
COMMAND(rsm, 0){ 
  String_write(term_reply, "NOT SUPPORTED: %s (wrong application)", str);
}                    

// ssm xx <br> Writes sensor mask (appconfig_t.sensorMask) to enables/disable sensors as a possible combination of SENSOR_VIBRATION, SENSOR_MOVEMENT, SENSOR_BUTTON, SENSOR_MICROPHONE, SENSOR_TEMPERATURE. 
COMMAND(ssm, 0){ 
  String_write(term_reply, "NOT SUPPORTED: %s (wrong application)", str);
} 

bool Event_init(){
  System_registerCallback(C_SENSOR, handleEvent);
  return True;
}
