/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

#include "ScatterWeb.RFC.h"

UINT16 RFCID;
UINT8 RFCHEADER[72];
UINT8 RFCDATA[72];

void RFC_doBool(bool b) {
  if(RFCID < 10) {
    if(b) String_write(term_reply, "rfc 00%i True", RFCID);
    else String_write(term_reply, "rfc 00%i False", RFCID);
  }
  else {
    if(RFCID < 100) {
      if(b) String_write(term_reply, "rfc 0%i True", RFCID);
      else String_write(term_reply, "rfc 0%i False", RFCID);
    }
    else {
      if(b) String_write(term_reply, "rfc %i True", RFCID);
      else String_write(term_reply, "rfc %i False", RFCID);
    }
  }
}

void RFC_doTemperature(temp_t t) {
  if(RFCID < 10) String_write(term_reply, "rfc 00%i %T", RFCID, t);
  else if(RFCID < 100) String_write(term_reply, "rfc 0%i %T", RFCID, t);
  else String_write(term_reply, "rfc %i %T", RFCID, t);
}

void RFC_doUINTPointer(UINT8* s) {
  if(RFCID < 10) String_write(term_reply, "rfc 00%i %s", RFCID, s);
  else if(RFCID < 100) String_write(term_reply, "rfc 0%i %s", RFCID, s);
  else String_write(term_reply, "rfc %i %s", RFCID, s);
  
}

void RFC_doUINT(UINT16 i) {
  if(RFCID < 10) String_write(term_reply, "rfc 00%i %i", RFCID, i);
  else if(RFCID < 100) String_write(term_reply, "rfc 0%i %i", RFCID, i);
  else String_write(term_reply, "rfc %i %i", RFCID, i);
}

void RFC_doLUINT(UINT32 i) {
  if(RFCID < 10) String_write(term_reply, "rfc 00%i %l", RFCID, i);
  else if(RFCID < 100) String_write(term_reply, "rfc 0%i %l", RFCID, i);
  else String_write(term_reply, "rfc %i %l", RFCID, i);
}

void RFC_doTime(time_t* t) {
  if(RFCID < 10) String_write(term_reply, "rfc 00%i %l %i", RFCID, t->secs, t->millis);
  else if(RFCID < 100) String_write(term_reply, "rfc 0%i %l %i", RFCID, t->secs, t->millis);
  else String_write(term_reply, "rfc %i %l %i", RFCID, t->secs, t->millis);
}

void  RFC_doDone() {
  if(RFCID < 10) String_write(term_reply, "rfc 00%i done", RFCID);
  else if(RFCID < 100) String_write(term_reply, "rfc 0%i done", RFCID);
  else String_write(term_reply, "rfc %i done", RFCID);
}

bool RFC_init() {
  Configuration.rxReceiveLimit = 65535;
  Configuration.transmitPower = 99;
  Configuration_save();
  Data_batteryOff();
  Data_beeperOff();
  Data_buttonOff();
  Data_greenOff();
  Data_redOff();
  Data_yellowOff();
  Data_microphoneOff();
  Data_movementOff();
  Data_temperatureOff();
  Data_vibrationOff();
  Data_RC5Disable();
  return True;
}

COMMAND(rfc, 0) {
  RFCID = String_parseInt(&str[4], NULL);
  switch(RFCID) {
      case RFC_INIT: RFC_doBool(RFC_init()); break;
      case COMM_PRINT: Comm_print(&str[8]); break;
      case COMM_LOG: Comm_log(String_parseInt(&str[8], NULL), &str[14]); break;
      case COMM_ON: RFC_doBool(Comm_on()); break;
      case COMM_OFF: RFC_doBool(Comm_off()); break;
      case COMM_REGISTERBYTELEVEL: RFC_doBool(Comm_registerByteLevel((fp_char_t) String_parseInt(&str[8], NULL))); break;
      case COMM_CLEAR: RFC_doBool(Comm_clear()); break;
      case CONFIGURATION_INIT: RFC_doBool(Configuration_init()); break;
      case CONFIGURATION_SAVE: RFC_doBool(Configuration_save()); break;
      case CONFIGURATION_PRINT: RFC_doBool(Configuration_print()); break;
      case DATA_INIT: RFC_doBool(Data_init()); break;
      case DATA_BATTERYON: RFC_doBool(Data_batteryOn(String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL))); break;
      case DATA_BATTERYOFF: RFC_doBool(Data_batteryOff()); break;
      case DATA_BEEPERON: RFC_doBool(Data_beeperOn()); break;
      case DATA_BEEPEROFF: RFC_doBool(Data_beeperOff()); break;
      case DATA_BEEPERTOGGLE: RFC_doBool(Data_beeperToggle()); break;
      case DATA_BUTTONON: RFC_doBool(Data_buttonOn(String_parseInt(&str[8], NULL))); break;
      case DATA_BUTTONOFF: RFC_doBool(Data_buttonOff()); break;
      case DATA_GREENON: RFC_doBool(Data_greenOn()); break;
      case DATA_GREENOFF: RFC_doBool(Data_greenOff()); break;
      case DATA_GREENTOGGLE: RFC_doBool(Data_greenToggle()); break;
      case DATA_REDON: RFC_doBool(Data_redOn()); break;
      case DATA_REDOFF: RFC_doBool(Data_redOff()); break;
      case DATA_REDTOGGLE: RFC_doBool(Data_redToggle()); break;
      case DATA_YELLOWON: RFC_doBool(Data_yellowOn()); break;
      case DATA_YELLOWOFF: RFC_doBool(Data_yellowOff()); break;
      case DATA_YELLOWTOGGLE: RFC_doBool(Data_yellowToggle()); break;
      case DATA_MICROPHONEON: RFC_doBool(Data_microphoneOn(String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL))); break;
      case DATA_MICROPHONEOFF: RFC_doBool(Data_microphoneOff()); break;
      case DATA_MOVEMENTON: RFC_doBool(Data_movementOn(String_parseInt(&str[8], NULL))); break;
      case DATA_MOVEMENTOFF: RFC_doBool(Data_movementOff()); break;
      case DATA_TEMPERATUREON: RFC_doBool(Data_temperatureOn(String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL), String_parseInt(&str[20], NULL), String_parseInt(&str[26], NULL))); break;
      case DATA_TEMPERATUREOFF: RFC_doBool(Data_temperatureOff()); break;
      case DATA_VIBRATIONON: RFC_doBool(Data_vibrationOn(String_parseInt(&str[8], NULL))); break;
      case DATA_VIBRATIONOFF: RFC_doBool(Data_vibrationOff()); break;
      case DATA_GREENLED: RFC_doBool(Data_greenLed()); break;
      case DATA_REDLED: RFC_doBool(Data_redLed()); break;
      case DATA_YELLOWLED: RFC_doBool(Data_yellowLed()); break;
      case DATA_RC5SENDPHI: RFC_doBool(Data_RC5SendPHI(String_parseInt(&str[8], NULL))); break;
      case DATA_RC5ENABLE: RFC_doBool(Data_RC5Enable()); break;
      case DATA_RC5DISABLE: RFC_doBool(Data_RC5Disable()); break;
      case DATA_RC5SENDNEC: RFC_doBool(Data_RC5SendNEC(String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL))); break;
      case DATA_SENSORHANDLER: RFC_doBool(Data_sensorHandler()); break;
      case DATA_RC5RECEIVEHANDLER: Data_RC5ReceiveHandler(); break;
      case DATA_RC5EVENTHANDLER: Data_RC5EventHandler(); break;
      case IO_ERASE: RFC_doBool(IO_erase(String_parseInt(&str[8], NULL), 1)); break;
      case IO_READ: {
        UINT8 i[1];
        RFC_doUINT((UINT16) IO_read(String_parseInt(&str[8], NULL), i, 1));
      }
      break;
      case IO_WRITE: RFC_doBool(IO_write(String_parseInt(&str[8], NULL), (UINT8*) String_parseInt(&str[14], NULL), 1)); break;
      case MESSAGING_ON: RFC_doBool(Messaging_on()); break;
      case MESSAGING_SERIALHANDLER: Messaging_serialHandler(); break;
      case MESSAGING_REQUESTHANDLER: RFC_doBool(Messaging_requestHandler()); break;
      case MESSAGING_ANSWERHANDLER: RFC_doBool(Messaging_answerHandler()); break;
      case MESSAGING_REQUESTDATAHANDLER: RFC_doBool(Messaging_requestDataHandler((UINT8*) &str[8])); break;
      case NET_INIT: RFC_doBool(Net_init()); break;
      case NET_ON: RFC_doBool(Net_on()); break;
      case NET_OFF: RFC_doBool(Net_off()); break;
      case NET_SETTXPOWER: RFC_doBool(Net_setTxPower(String_parseInt(&str[8], NULL))); break;
      case NET_SEND: {
        packet_t p;
        p.to = String_parseInt(&str[8], NULL);
        p.from = String_parseInt(&str[14], NULL);
        p.type = String_parseInt(&str[20], NULL);
        p.num = String_parseInt(&str[26], NULL);
        p.header_length = String_parseInt(&str[32], NULL);
        p.data_length = String_parseInt(&str[38], NULL);
        p.header = RFCHEADER;
        p.data = RFCDATA;
        RFC_doBool(Net_send(&p, (fp_char_t) String_parseInt(&str[44], NULL)));
      }
      break;
      case NET_RXHANDLER: RFC_doBool(Net_rxHandler()); break;
      case NET_TXHANDLER: RFC_doBool(Net_txHandler()); break;
      case NET_ESEND: {
        packet_t p;
        p.to = String_parseInt(&str[8], NULL);
        p.from = String_parseInt(&str[14], NULL);
        p.type = String_parseInt(&str[20], NULL);
        p.num = String_parseInt(&str[26], NULL);
        p.header_length = String_parseInt(&str[32], NULL);
        p.data_length = String_parseInt(&str[38], NULL);
        p.header = RFCHEADER;
        p.data = RFCDATA;
        RFC_doBool(Net_esend(&p, (fp_char_t) String_parseInt(&str[44], NULL)));
      }
      break;
      case STRING_WRITEUINT: RFC_doUINT(String_writeUInt((fp_char_t) String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL), String_parseInt(&str[22], NULL))); break;
      case STRING_WRITEULONGINT: RFC_doUINT(String_writeULongInt((fp_char_t) String_parseInt(&str[8], NULL), String_parseLongInt(&str[20], NULL), String_parseInt(&str[14], NULL))); break;
      case STRING_FROMBYTETOHEX: RFC_doBool(String_fromByteToHex(str[8], (UINT8*) &str[10])); break;
      case STRING_FROMHEXTOBYTE: RFC_doUINT(String_fromHexToByte((UINT8*) &str[8])); break;
      case STRING_WRITETEMPERATURE: {
        temp_t t;
        t.u = String_parseInt(&str[14], NULL);
        t.s = String_parseInt(&str[22], NULL);
        RFC_doBool(String_writeTemperature((fp_char_t) String_parseInt(&str[8], NULL), t));
      }
      break;
      case STRING_ABSTRACTWRITE: RFC_doBool(String_abstractWrite((fp_char_t) String_parseInt(&str[14], NULL), "%s", (va_list) &str[22])); break;
      case STRING_WRITE: RFC_doBool(String_write((UINT8*) String_parseInt(&str[14], NULL), &str[16])); break;
      case STRING_PARSEINT: RFC_doUINT(String_parseInt(&str[8], NULL)); break;
      case STRING_PARSELONGINT: RFC_doLUINT(String_parseLongInt(&str[8], NULL)); break;
      case SYNCHRONISATION_INIT: RFC_doBool(Synchronisation_init()); break;
      case SYNCHRONISATION_ISROOT: RFC_doBool(Synchronisation_isRoot()); break;
      case SYNCHRONISATION_REGISTERSYNCDONE: RFC_doBool(Synchronisation_registerSyncDone((fp_t) String_parseInt(&str[8], NULL))); break;
      case SYNCHRONISATION_DOSYNC: RFC_doBool(Synchronisation_doSync()); break;
      case SYSTEM_REGISTERCALLBACK: RFC_doBool(System_registerCallback(String_parseInt(&str[8], NULL), (fp_t) String_parseInt(&str[14], NULL))); break;
      case SYSTEM_GETTEMPERATURE: RFC_doTemperature(System_getTemperature()); break;
      case SYSTEM_SETDCO: RFC_doBool(System_setDCO()); break;
      case SYSTEM_STARTWATCHDOG: RFC_doBool(System_startWatchdog()); break;
      case SYSTEM_STOPWATCHDOG: RFC_doBool(System_stopWatchdog()); break;
      case SYSTEM_RESET: RFC_doBool(System_reset()); break;
      case SYSTEM_RESETREASON: RFC_doUINTPointer((UINT8*) System_resetReason()); break;
      case SYSTEM_PANIC: RFC_doBool(System_panic(&str[8])); break;
      case SYSTEM_PANICREASON: RFC_doUINTPointer(System_panicReason(String_parseInt(&str[8], NULL))); break;
      case SYSTEM_PANICCLEAR: RFC_doBool(System_panicClear()); break;
      case SYSTEM_WAIT: RFC_doBool(System_wait(String_parseLongInt(&str[8], NULL))); break;
      case SYSTEM_WAITA: RFC_doBool(System_waitA(String_parseInt(&str[8], NULL))); break;
      case SYSTEM_POWERON: RFC_doBool(System_powerOn()); break;
      case SYSTEM_POWEROFF: RFC_doBool(System_powerOff()); break;
      case SYSTEM_REPROGRAMM: RFC_doUINT((UINT16) System_reprogram()); break;
      case SYSTEM_STARTCONVERSION: RFC_doBool(System_startConversion()); break;
      case THREADING_INIT: RFC_doBool(Threading_init()); break;
      case THREADING_ADD: RFC_doBool(Threading_add((void*) String_parseInt(&str[8], NULL))); break;
      case THREADING_DOEVENTS: RFC_doBool(Threading_doEvents()); break;
      case THREADING_SLEEP: RFC_doBool(Threading_sleep(String_parseInt(&str[8], NULL))); break;
      case THREADING_WAIT: RFC_doBool(Threading_wait((UINT8*) String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL))); break;
      case THREADING_EVENTHANDLER: RFC_doBool(Threading_eventHandler()); break;
      case THREADING_PACKETHANDLER: RFC_doBool(Threading_packetHandler()); break;
      case THREADING_SENTPACKETHANDLER: RFC_doBool(Threading_sentPacketHandler(String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL))); break;
      case THREADING_RECV: RFC_doBool(Threading_recv(String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL), String_parseInt(&str[20], NULL))); break;
      case THREADING_RECVP:  RFC_doBool(Threading_recvp(String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL), String_parseInt(&str[20], NULL))); break;
      case THREADING_SEND: RFC_doBool(Threading_send(String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL), RFCHEADER, String_parseInt(&str[20], NULL), RFCDATA, String_parseInt(&str[26], NULL))); break;
      case THREADING_ESEND: RFC_doBool(Threading_esend(String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL), RFCHEADER, String_parseInt(&str[20], NULL), (UINT16) RFCDATA, String_parseInt(&str[26], NULL))); break;
      case TIME_GETSYSTEMTIME: {
        time_t t;
        t.secs = 0;
        t.millis = 0;
        RFC_doTime(Time_getSystemTime(&t));
      }
      break;
      case TIME_SETSYSTEMTIME: RFC_doBool(Time_setSystemTime((time_t*) String_parseInt(&str[8], NULL))); break;
      case TIME_CONVERTSYSTEMTIME: RFC_doBool(Time_convertSystemTime((time_t*) String_parseInt(&str[8], NULL), (btime_t*) String_parseInt(&str[14], NULL))); break;
      case TIME_CONVERTTIME: RFC_doBool(Time_convertTime((btime_t*) String_parseInt(&str[8], NULL), (time_t*) String_parseInt(&str[14], NULL))); break;
      case TIME_WRITETIME: RFC_doBool(Time_writeTime((fp_char_t) String_parseInt(&str[8], NULL), (btime_t*) String_parseInt(&str[14], NULL))); break;
      case TIME_READTIME: RFC_doBool(Time_readTime((UINT8*) String_parseInt(&str[8], NULL), (btime_t*) String_parseInt(&str[14], NULL))); break;
      case TIMERS_INIT: RFC_doBool(Timers_init()); break;
      case TIMERS_ADD: RFC_doBool(Timers_add(String_parseInt(&str[8], NULL), (fp_timer) String_parseInt(&str[14], NULL), String_parseInt(&str[20], NULL))); break;
      case TIMERS_RESET: RFC_doBool(Timers_reset(String_parseInt(&str[8], NULL), (fp_timer) String_parseInt(&str[14], NULL), String_parseInt(&str[20], NULL))); break;
      case TIMERS_REMOVE: RFC_doBool(Timers_remove((fp_timer) String_parseInt(&str[8], NULL), String_parseInt(&str[14], NULL))); break;
      case TIMERS_EVENTHANDLER: RFC_doBool(Timers_eventHandler()); break;
      case PACKET_HEADER: {
        int i;
        for(i = 0; i<72; i++) RFCHEADER[i] = str[i+8];
        RFC_doDone();
      }
      break;
      case PAKCET_DATA: {
        int i;
        for(i = 0; i<72; i++) RFCDATA[i] = str[i+8];
        RFC_doDone();
      }
      break;
      default: String_write(term_reply, "rfc error"); break;
  }
  Comm_clear();
}
