/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

/** \file ScatterWeb.String.h
 ** \ingroup System
 ** \brief The ScatterWeb.String represents text; that is, a series of Unicode
 ** characters.
 **
 ** @author        Marco Hutta
 ** @author        Bjoern Lichtblau
 ** @version       2.3
 ** @since         2.0     
 **/

#ifndef ScatterWeb_String
#define ScatterWeb_String

/**
 * Converts int to String
 *
 * @param writer    function to write
 * @param number    number to write
 * @param length    length to write
 * @return          position in output out[pos].
 * @since           2.0
 */
int String_writeUInt(fp_char_t writer, UINT16 number, UINT16 length);

/**
 * Converts long int to String
 *
 * @param writer    function to write
 * @param number    number to write
 * @param length    length to write
 * @return          position in output out[pos].
 * @since           2.0
 */
int String_writeULongInt(fp_char_t writer, UINT32 number, UINT16 length);

/**
 * Converts the byte *b into the string cs[0] cs[1].
 *
 * @param b         byte
 * @param cs        string
 * @return          <code>True</code> always.
 * @since           2.0
 */
bool String_fromByteToHex(UINT8 b, UINT8* cs);

/**
 * Converts the string at cs[0] cs[1] into a byte at *b.
 *
 * @param cs        string
 * @return          value to convert into.
 * @since           2.0
 */
UINT8 String_fromHexToByte(const UINT8* cs);

/**
 * Writes a temperature according to the format described in 
 * System_getTemperature ([+-]xxx.x).
 *
 * @param writer    function to write
 * @param temp      temperature
 * @return          <code>True</code> always.
 * @since           2.0
 */
bool String_writeTemperature(fp_char_t writer, temp_t temp);

/**
 * Simple printf implementation for RS232 output.
 * Supports - amongst others:
 *  - %c: Prints a character.
 *  - %i: Prints an UINT16eger in decimal format.
 *  - %l: Prints an UINT32eger in decimal format
 *  - %b: Prints a byte in hex format.
 *  - %x: Prints an int in hex format. 
 *
 * @param writer    function to write
 * @param fmt       function
 * @param argp      list of arguments
 * @return          <code>True</code> always.
 * @since           2.0
 */
bool String_abstractWrite(fp_char_t writer, const char *fmt, va_list argp);

/**
 * Writes
 *
 * @param dest      dest
 * @param fmt       function
 * @param ...       list of parameters
 * @return          <code>True</code> always.
 * @see             String_abstractWrite
 * @since           2.0
 */
bool String_write(UINT8* dest, const char *fmt, ...);

/**
 * Parses int
 *
 * @param c         c
 * @param ptr       ptr
 * @return UINT16   parsed int  
 * @since           2.0
 */
UINT16 String_parseInt(const UINT8* c, UINT8** ptr);

/**
 * Parses long int
 *
 * @param c         c
 * @param ptr       ptr
 * @return UINT16   parsed long int  
 * @since           2.0
 */
UINT32 String_parseLongInt(const UINT8* c, UINT8** ptr);

#endif
