/*
Copyright 2005, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2005
*/

/** \file ScatterWeb.Synchronisation.h
 ** \ingroup System
 ** \brief The ScatterWeb.Synchronisation represents the synchronisation.
 **
 ** @author        Marco Hutta
 ** @version       2.3
 ** @since         2.2
 **/  

#ifndef ScatterWeb_Synchronisation
#define ScatterWeb_Synchronisation

/**
 * The number of sync events without new message from root, till this node
 * declares itself to be the root
 *
 * @since         2.2
 */
#define ROOT_TIMEOUT          5
                                    
/**
 * In minutes; higher values will cause increasingly lost precision this works
 * without drift compensation
 *
 * @since         2.2
 */
#define FRAME_LENGTH          3

/**
 * The time period in minutes before sending a sync messages; keep this HIGHER
 * than the frequency to allow adapting to a present net before sending 
 *
 * @since         2.2
 */
#define INIT_LENGTH           4

/**
 * Synchronisation struct
 *
 * @since         2.2
 */
volatile struct {
  UINT8 seqNum;
  UINT16 masterRootId;
  UINT8 syncPeriods;
} SyncStruct;

/**
 * Starts the synchronisation (not started by system)
 *
 * @return        <code>True</code>  always.
 * @since         2.2
 */
bool Synchronisation_init();

/**
 * Checks if node is currently the root
 *
 * @return        <code>True</code> if node is root;
 *                <code>False</code> otherwise.
 * @since         2.2
 */
bool Synchronisation_isRoot();

/**
 * Registers a function which is called if the sync ends
 *
 * @param ()      function to call 
 * @return        <code>True</code> always.
 * @since         2.2
 */
bool Synchronisation_registerSyncDone(void(*fp_t)());

/* 
 * This member supports the ScatterWeb infrastructure
 * and is not intended to be used directly from your code.
 */
bool Synchronisation_doSync();

#endif
